import os
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from sunpy import map as smap
import sunpy.cm.cm as cm_smap
from suncasa.utils import plot_mapX as pmX
import astropy.units as u
from mpl_toolkits.axes_grid1 import make_axes_locatable
import matplotlib.colorbar as colorbar
import matplotlib.patches as patches
from datetime import timedelta
from datetime import datetime
from glob import glob
import numpy as np
from astropy.time import Time
import urllib.request

imgfitsdir = '/data1/eovsa/fits/synoptic/'
imgfitstmpdir = '/data1/workdir/fitstmp/'
pltfigdir = '/common/webplots/SynopticImg/eovsamedia/eovsa-browser/'


def clearImage():
    for (dirpath, dirnames, filenames) in os.walk(pltfigdir):
        for filename in filenames:
            for k in ['0094', '0193', '0335', '4500', '0171', '0304', '0131', '1700', '0211', '1600', '_HMIcont',
                      '_HMImag']:
                # for k in ['_Halph_fr']:
                if k in filename:
                    print(os.path.join(dirpath, filename))
                    os.system('rm -rf ' + os.path.join(dirpath, filename))


def pltEovsaQlookImage(timobj, spws, vmaxs, vmins, dpis_dict, fig=None, ax=None, overwrite=False, verbose=False):
    plt.ioff()
    dateobj = timobj.to_datetime()
    datestrdir = dateobj.strftime("%Y/%m/%d/")
    imgindir = imgfitsdir + datestrdir

    # if not os.path.exists(imgindir):
    #     os.makedirs(imgindir)
    cmap = cm_smap.get_cmap('sdoaia304')

    if fig is None or ax is None:
        mkfig = True
    else:
        mkfig = False

    if mkfig:
        fig, ax = plt.subplots(figsize=(8, 8))
        fig.subplots_adjust(bottom=0.0, top=1.0, left=0.0, right=1.0)

    if verbose: print('Processing EOVSA images for date {}'.format(dateobj.strftime('%Y-%m-%d')))
    for s, sp in enumerate(spws):
        fexists = []
        for l, dpi in dpis_dict.items():
            figname = os.path.join(movieoutdir, '{}_eovsa_bd{:02d}.jpg'.format(l, s + 1))
            fexists.append(os.path.exists(figname))

        if overwrite or (False in fexists):
            ax.cla()
            spwstr = '-'.join(['{:02d}'.format(int(sp_)) for sp_ in sp.split('~')])
            eofile = imgindir + 'eovsa_{}.spw{}.tb.disk.fits'.format(dateobj.strftime('%Y%m%d'), spwstr)
            if not os.path.exists(eofile): continue
            if not os.path.exists(movieoutdir): os.makedirs(movieoutdir)
            eomap = smap.Map(eofile)
            norm = colors.Normalize(vmin=vmins[s], vmax=vmaxs[s])
            eomap_ = pmX.Sunmap(eomap)
            eomap_.imshow(axes=ax, cmap=cmap, norm=norm)
            eomap_.draw_limb(axes=ax, lw=0.5, alpha=0.5)
            eomap_.draw_grid(axes=ax, grid_spacing=10. * u.deg, lw=0.5)
            ax.set_xlabel('')
            ax.set_ylabel('')
            ax.set_xticklabels([])
            ax.set_yticklabels([])
            ax.text(0.02, 0.02,
                    'EOVSA {:.1f} GHz  {}'.format(eomap.meta['CRVAL3'] / 1e9, eomap.date.strftime('%d-%b-%Y 20:00 UT')),
                    transform=ax.transAxes, color='w', ha='left', va='bottom', fontsize=9)
            ax.text(0.98, 0.02, 'Max Tb {:.0f} K'.format(np.nanmax(eomap.data)),
                    transform=ax.transAxes, color='w', ha='right', va='bottom', fontsize=9)
            ax.set_xlim(-1227, 1227)
            ax.set_ylim(-1227, 1227)

            for l, dpi in dpis_dict.items():
                figname = os.path.join(movieoutdir, '{}_eovsa_bd{:02d}.jpg'.format(l, s + 1))
                fig.savefig(figname, dpi=np.int(dpi), quality=85)
    if mkfig:
        pass
    else:
        plt.close(fig)
    return


def pltSdoQlookImage(timobj, dpis_dict, fig=None, ax=None, overwrite=False, verbose=False, clearcache=False):
    plt.ioff()
    dateobj = timobj.to_datetime()
    datestrdir = dateobj.strftime("%Y/%m/%d/")
    monthstrdir = dateobj.strftime("%Y/%m/")
    imgindir = imgfitsdir + datestrdir
    imgoutdir = imgfitstmpdir + monthstrdir
    movieoutdir = pltfigdir + monthstrdir

    if not os.path.exists(imgindir):
        os.makedirs(imgindir)
    if not os.path.exists(movieoutdir):
        os.makedirs(movieoutdir)

    aiaDataSource = {"0094": 8,
                     "0193": 11,
                     "0335": 14,
                     # "4500": 17,
                     "0171": 10,
                     "0304": 13,
                     "0131": 9,
                     "1700": 16,
                     "0211": 12,
                     # "1600": 15,
                     "_HMIcont": 18,
                     "_HMImag": 19}

    if fig is None or ax is None:
        mkfig = True
    else:
        mkfig = False

    if mkfig:
        fig, ax = plt.subplots(figsize=(8, 8))
        fig.subplots_adjust(bottom=0.0, top=1.0, left=0.0, right=1.0)

    if verbose: print('Processing SDO images for date {}'.format(dateobj.strftime('%Y-%m-%d')))
    for key, sourceid in aiaDataSource.items():
        fexists = []
        for l, dpi in dpis_dict.items():
            figname = os.path.join(imgoutdir, '{}{}.jpg'.format(l, key))
            fexists.append(os.path.exists(figname))

        if overwrite or (False in fexists):
            sdourl = 'https://api.helioviewer.org/v2/getJP2Image/?date={}T20:00:00Z&sourceId={}'.format(timobj,
                                                                                                        sourceid)
            sdofile = os.path.join(imgindir, key + '.jp2')
            if not os.path.exists(sdofile):
                urllib.request.urlretrieve(sdourl, sdofile)
            ax.cla()

            if not os.path.exists(sdofile): continue
            if not os.path.exists(imgoutdir): os.makedirs(imgoutdir)
            sdomap = smap.Map(sdofile)
            norm = colors.Normalize()
            sdomap_ = pmX.Sunmap(sdomap)
            if "HMI" in key:
                cmap = plt.get_cmap('gray')
            else:
                cmap = cm_smap.get_cmap('sdoaia' + key.lstrip('0'))
            sdomap_.imshow(axes=ax, cmap=cmap, norm=norm)
            sdomap_.draw_limb(axes=ax, lw=0.5, alpha=0.5)
            sdomap_.draw_grid(axes=ax, grid_spacing=10. * u.deg, lw=0.5)
            ax.set_xlabel('')
            ax.set_ylabel('')
            ax.set_xticklabels([])
            ax.set_yticklabels([])
            ax.text(0.02, 0.02,
                    '{}/{} {}  {}'.format(sdomap.observatory, sdomap.instrument.split(' ')[0], sdomap.measurement,
                                          sdomap.date.strftime('%d-%b-%Y %H:%M UT')),
                    transform=ax.transAxes, color='w', ha='left', va='bottom', fontsize=9)
            ax.set_xlim(-1227, 1227)
            ax.set_ylim(-1227, 1227)

            for l, dpi in dpis_dict.items():
                figname = os.path.join(imgoutdir, '{}{}.jpg'.format(l, key))
                fig.savefig(figname, dpi=np.int(dpi), quality=85)
    if clearcache:
        os.system('rm -rf ' + imgindir)

    if mkfig:
        pass
    else:
        plt.close(fig)
    return


def pltBbsoQlookImage(datestr, dpis_dict, fig=None, ax=None, overwrite=False, verbose=False, clearcache=False):
    from astropy.io import fits
    from html.parser import HTMLParser
    class MyHTMLParser(HTMLParser):
        def __init__(self, prefix='bbso_halph_fr_', suffix='.fts'):
            HTMLParser.__init__(self)
            self.prefix = prefix
            self.suffix = suffix

        def handle_starttag(self, tag, attrs):
            if tag != 'a':
                return
            for name, value in attrs:
                if name == "href":
                    if value.startswith(self.prefix) and value.endswith(self.suffix):
                        self.links.append(value)

    def extract(url, prefix='bbso_halph_fr_', suffix='.fts'):
        import urllib.request
        with urllib.request.urlopen(url) as response:
            f = response.read()

        parser = MyHTMLParser(prefix, suffix)
        parser.links = []
        parser.feed(str(f))
        return parser.links

    bbsodir = 'http://www.bbso.njit.edu/pub/archive/'
    plt.ioff()
    dateobj = datetime.strptime(datestr, "%Y-%m-%d")
    datestrdir = dateobj.strftime("%Y/%m/%d/")
    imgindir = os.path.join(imgfitstmpdir, datestr)
    imgoutdir = pltfigdir + datestrdir
    if not os.path.exists(imgindir):
        os.makedirs(imgindir)

    bbsoDataSource = {"_Halph_fr": ["bbso_halph_fr_", ".fts"]}

    if fig is None or ax is None:
        mkfig = True
    else:
        mkfig = False

    if mkfig:
        fig, ax = plt.subplots(figsize=(8, 8))
        fig.subplots_adjust(bottom=0.0, top=1.0, left=0.0, right=1.0)

    if verbose: print('Processing BBSO images for date {}'.format(dateobj.strftime('%Y-%m-%d')))
    for key, sourceid in bbsoDataSource.items():
        fexists = []
        for l, dpi in dpis_dict.items():
            figname = os.path.join(imgoutdir, '{}{}.jpg'.format(l, key))
            fexists.append(os.path.exists(figname))

        if overwrite or (False in fexists):
            bbsosite = os.path.join(bbsodir, datestrdir)
            filelist = extract(bbsosite, sourceid[0], sourceid[1])
            if filelist:
                tfilelist = Time(
                    [datetime.strptime(tf.replace(sourceid[0], '').replace(sourceid[1], ''), "%Y%m%d_%H%M%S") for tf in
                     filelist])
                bbsourl = os.path.join(bbsosite, filelist[
                    np.nanargmin(np.abs(np.array(tfilelist.mjd - (Time(dateobj).mjd + 20. / 24.))))])

                bbsofile = os.path.join(imgindir, key + '.fits')
                if not os.path.exists(bbsofile):
                    urllib.request.urlretrieve(bbsourl, bbsofile)
                ax.cla()
                if not os.path.exists(bbsofile): continue
                if not os.path.exists(imgoutdir): os.makedirs(imgoutdir)
                hdu = fits.open(bbsofile)[0]
                header = hdu.header
                header['WAVELNTH'] = 6562.8
                header['WAVEUNIT'] = 'angstrom'
                header['WAVE_STR'] = 'Halph'
                header['CTYPE1'] = 'HPLN-TAN'
                header['CUNIT1'] = 'arcsec'
                header['CTYPE2'] = 'HPLT-TAN'
                header['CUNIT2'] = 'arcsec'
                header['DATE-OBS'] = header['DATE_OBS']
                for k in ['CONTRAST', 'WAVE ERR']:
                    try:
                        header.remove(k)
                    except:
                        pass

                bbsomap = smap.Map(hdu.data, header)
                med = np.nanmean(bbsomap.data)
                norm = colors.Normalize(vmin=med - 1500, vmax=med + 1500)
                bbsomap_ = pmX.Sunmap(bbsomap)
                cmap = cm_smap.get_cmap('sdoaia304')
                bbsomap_.imshow(axes=ax, cmap=cmap, norm=norm)
                bbsomap_.draw_limb(axes=ax, lw=0.5, alpha=0.5)
                bbsomap_.draw_grid(axes=ax, grid_spacing=10. * u.deg, lw=0.5)
                ax.set_xlabel('')
                ax.set_ylabel('')
                ax.set_xticklabels([])
                ax.set_yticklabels([])
                ax.text(0.02, 0.02,
                        '{}  {}'.format(bbsomap.instrument, bbsomap.date.strftime('%d-%b-%Y %H:%M UT')),
                        transform=ax.transAxes, color='w', ha='left', va='bottom', fontsize=9)
                ax.set_xlim(-1227, 1227)
                ax.set_ylim(-1227, 1227)
                ax.set_facecolor('k')

                for l, dpi in dpis_dict.items():
                    figname = os.path.join(imgoutdir, '{}{}.jpg'.format(l, key))
                    fig.savefig(figname, dpi=np.int(dpi), quality=85)
    if clearcache:
        os.system('rm -rf ' + imgindir)

    if mkfig:
        pass
    else:
        plt.close(fig)
    return


def main(year=None, month=None, day=None, dayspan=30, clearcache=False):
    '''
    By default, the subroutine create EOVSA monthly movie
    '''
    # tst = datetime.strptime("2017-04-01", "%Y-%m-%d")
    # ted = datetime.strptime("2019-12-31", "%Y-%m-%d")
    if year:
        ted = datetime(year, month, day)
    else:
        ted = datetime.now() - timedelta(days=2)
    tst = Time(np.fix(Time(ted).mjd) - dayspan, format='mjd').datetime
    tsep = datetime.strptime('2019-02-22', "%Y-%m-%d")

    vmaxs = [22.0e4, 8.0e4, 5.4e4, 3.5e4, 2.3e4, 1.8e4, 1.5e4]
    vmins = [-9.0e3, -5.5e3, -3.4e3, -2.5e3, -2.5e3, -2.5e3, -2.5e3]

    dpis = np.array([256, 512, 1024]) / 8
    dpis_dict_eo = {'t': dpis[0], 'l': dpis[1], 'f': dpis[2]}
    dpis = np.array([256, 512, 2048]) / 8
    dpis_dict_sdo = {'t': dpis[0], 'l': dpis[1], 'f': dpis[2]}
    dpis = np.array([256, 512, 2048]) / 8
    dpis_dict_bbso = {'t': dpis[0], 'l': dpis[1], 'f': dpis[2]}

    plt.ioff()
    fig, ax = plt.subplots(nrows=1, ncols=2, figsize=(8, 4), sharex=True, sharey=True)
    fig.subplots_adjust(bottom=0.0, top=1.0, left=0.0, right=1.0)

    dateobs = tst
    while dateobs < ted:
        dateobs = dateobs + timedelta(days=1)
        monthstrdir = dateobs.strftime("%Y/%m/")
        imgoutdir = imgfitstmpdir + monthstrdir
        movieoutdir = pltfigdir + monthstrdir
        for odir in [imgoutdir,movieoutdir]:
            if not os.path.exists(odir):
                os.makedirs(odir)

        # dateobs = datetime.strptime("2019-12-21", "%Y-%m-%d")

        if dateobs > tsep:
            spws = ['0~1', '2~5', '6~10', '11~20', '21~30', '31~43', '44~49']
        else:
            spws = ['1~3', '4~9', '10~16', '17~24', '25~30']

        for hr in range(0, 24, 2):
            tdateobs = Time(Time(dateobs).mjd + hr / 24, format='mjd')
            # datestr = tdateobs.strftime("%Y-%m-%d")
            pltEovsaQlookImage(tdateobs, spws, vmaxs, vmins, dpis_dict_eo, fig, ax, overwrite=False, verbose=True)
            pltSdoQlookImage(tdateobs, dpis_dict_sdo, fig, ax, overwrite=False, verbose=True, clearcache=clearcache)
            # pltBbsoQlookImage(datestr, dpis_dict_bbso, fig, ax, overwrite=False, verbose=True, clearcache=clearcache)
            # todo save images here

if __name__ == '__main__':
    import sys
    import numpy as np

    # import subprocess
    # shell = subprocess.check_output('echo $0', shell=True).decode().replace('\n', '').split('/')[-1]
    # print("shell " + shell + " is using")

    print(sys.argv)
    try:
        argv = sys.argv[1:]
        if '--clearcache' in argv:
            clearcache = True
            argv.remove('--clearcache')  # Allows --clearcache to be either before or after date items
        else:
            clearcache = False

        year = np.int(argv[0])
        month = np.int(argv[1])
        day = np.int(argv[2])
        if len(argv) == 3:
            dayspan = 30
        else:
            dayspan = np.int(argv[3])
    except:
        print('Error interpreting command line argument')
        year = None
        month = None
        day = None
        dayspan = 30
        clearcache = True
    print("Running pipeline_plt for date {}-{}-{}. clearcache {}".format(year, month, day, clearcache))
    main(year, month, day, dayspan, clearcache)



#
# This file was generated using xslt from its XML file
#
# Copyright 2009, Associated Universities Inc., Washington DC
#
import sys
import os
from  casac import *
import string
from taskinit import casalog
from taskinit import xmlpath
#from taskmanager import tm
import task_ptclean
def ptclean(vis='', imageprefix='', imagesuffix='', ncpu=8, twidth=1, doreg=False, usephacenter=False, reftime='', toTb=False, overwrite=False, outlierfile='', field='', spw='', selectdata=True, timerange='', uvrange='', antenna='', scan='', observation='', intent='', mode='mfs', resmooth=False, gridmode='', wprojplanes=-1, facets=1, cfcache='cfcache.dir', rotpainc=5.0, painc=360.0, aterm=True, psterm=False, mterm=True, wbawp=False, conjbeams=True, epjtable='', interpolation='linear', niter=500, gain=0.1, threshold='0.0mJy', psfmode='clark', imagermode='csclean', ftmachine='mosaic', mosweight=False, scaletype='SAULT', multiscale=[0], negcomponent=-1, smallscalebias=0.6, interactive=False, mask=[], nchan=-1, start=0, width=1, outframe='', veltype='radio', imsize=[256, 256], cell=['1.0arcsec'], phasecenter='', restfreq='', stokes='I', weighting='natural', robust=0.0, uvtaper=False, outertaper=[''], innertaper=['1.0'], modelimage='', restoringbeam=[''], pbcor=False, minpb=0.2, usescratch=False, noise='1.0Jy', npixels=0, npercycle=100, cyclefactor=1.5, cyclespeedup=-1, nterms=1, reffreq='', chaniter=False, flatnoise=True, allowchunk=False):

        """Parallelized clean in consecutive time steps
  
        """
        if type(multiscale)==int: multiscale=[multiscale]
        if type(imsize)==int: imsize=[imsize]
        if type(cell)==float: cell=[cell]
        if type(outertaper)==str: outertaper=[outertaper]
        if type(innertaper)==str: innertaper=[innertaper]
        if type(restoringbeam)==str: restoringbeam=[restoringbeam]

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['imageprefix'] = imageprefix
        mytmp['imagesuffix'] = imagesuffix
        mytmp['ncpu'] = ncpu
        mytmp['twidth'] = twidth
        mytmp['doreg'] = doreg
        mytmp['usephacenter'] = usephacenter
        mytmp['reftime'] = reftime
        mytmp['toTb'] = toTb
        mytmp['overwrite'] = overwrite
        mytmp['outlierfile'] = outlierfile
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['selectdata'] = selectdata
        mytmp['timerange'] = timerange
        mytmp['uvrange'] = uvrange
        mytmp['antenna'] = antenna
        mytmp['scan'] = scan
        mytmp['observation'] = observation
        mytmp['intent'] = intent
        mytmp['mode'] = mode
        mytmp['resmooth'] = resmooth
        mytmp['gridmode'] = gridmode
        mytmp['wprojplanes'] = wprojplanes
        mytmp['facets'] = facets
        mytmp['cfcache'] = cfcache
        mytmp['rotpainc'] = rotpainc
        mytmp['painc'] = painc
        mytmp['aterm'] = aterm
        mytmp['psterm'] = psterm
        mytmp['mterm'] = mterm
        mytmp['wbawp'] = wbawp
        mytmp['conjbeams'] = conjbeams
        mytmp['epjtable'] = epjtable
        mytmp['interpolation'] = interpolation
        mytmp['niter'] = niter
        mytmp['gain'] = gain
        if type(threshold) == str :
           mytmp['threshold'] = casac.quanta().quantity(threshold)
        else :
           mytmp['threshold'] = threshold
        mytmp['psfmode'] = psfmode
        mytmp['imagermode'] = imagermode
        mytmp['ftmachine'] = ftmachine
        mytmp['mosweight'] = mosweight
        mytmp['scaletype'] = scaletype
        mytmp['multiscale'] = multiscale
        mytmp['negcomponent'] = negcomponent
        mytmp['smallscalebias'] = smallscalebias
        mytmp['interactive'] = interactive
        mytmp['mask'] = mask
        mytmp['nchan'] = nchan
        mytmp['start'] = start
        mytmp['width'] = width
        mytmp['outframe'] = outframe
        mytmp['veltype'] = veltype
        mytmp['imsize'] = imsize
        if type(cell) == str :
           mytmp['cell'] = casac.quanta().quantity(cell)
        else :
           mytmp['cell'] = cell
        mytmp['phasecenter'] = phasecenter
        mytmp['restfreq'] = restfreq
        mytmp['stokes'] = stokes
        mytmp['weighting'] = weighting
        mytmp['robust'] = robust
        mytmp['uvtaper'] = uvtaper
        mytmp['outertaper'] = outertaper
        mytmp['innertaper'] = innertaper
        mytmp['modelimage'] = modelimage
        mytmp['restoringbeam'] = restoringbeam
        mytmp['pbcor'] = pbcor
        mytmp['minpb'] = minpb
        mytmp['usescratch'] = usescratch
        mytmp['noise'] = noise
        mytmp['npixels'] = npixels
        mytmp['npercycle'] = npercycle
        mytmp['cyclefactor'] = cyclefactor
        mytmp['cyclespeedup'] = cyclespeedup
        mytmp['nterms'] = nterms
        mytmp['reffreq'] = reffreq
        mytmp['chaniter'] = chaniter
        mytmp['flatnoise'] = flatnoise
        mytmp['allowchunk'] = allowchunk
	pathname="file:///Users/fisher/Dropbox/PycharmProjects/suncasa/tasks/"
	trec = casac.utils().torecord(pathname+'ptclean.xml')

        casalog.origin('ptclean')
        if trec.has_key('ptclean') and casac.utils().verify(mytmp, trec['ptclean']) :
	    result = task_ptclean.ptclean(vis, imageprefix, imagesuffix, ncpu, twidth, doreg, usephacenter, reftime, toTb, overwrite, outlierfile, field, spw, selectdata, timerange, uvrange, antenna, scan, observation, intent, mode, resmooth, gridmode, wprojplanes, facets, cfcache, rotpainc, painc, aterm, psterm, mterm, wbawp, conjbeams, epjtable, interpolation, niter, gain, threshold, psfmode, imagermode, ftmachine, mosweight, scaletype, multiscale, negcomponent, smallscalebias, interactive, mask, nchan, start, width, outframe, veltype, imsize, cell, phasecenter, restfreq, stokes, weighting, robust, uvtaper, outertaper, innertaper, modelimage, restoringbeam, pbcor, minpb, usescratch, noise, npixels, npercycle, cyclefactor, cyclespeedup, nterms, reffreq, chaniter, flatnoise, allowchunk)

	else :
	  result = False
        return result



import matplotlib.gridspec as gridspec
import numpy as np
import os
import datetime
import struct
from scipy.io.idl import readsav
from datetime import datetime
from taskinit import ms, tb, qa
import matplotlib.dates as mdates
from matplotlib.dates import date2num, AutoDateFormatter, AutoDateLocator


def get_dspec(vis=None, savespec=True, specfile=None, bl='', uvrange='', field='', scan='', datacolumn='data',
              domedian=False, timeran=None, spw=None, timebin='0s', verbose=False):
    from split_cli import split_cli as split

    msfile = vis
    if not spw:
        spw = ''
    if not timeran:
        timeran = ''
    if not bl:
        bl = ''
    if domedian:
        if not uvrange:
            uvrange = '0.2~0.8km'
    else:
        uvrange = ''
    # Open the ms and plot dynamic spectrum
    if verbose:
        print('Splitting selected data...')
    vis_spl = './tmpms.splitted'
    if os.path.exists(vis_spl):
        os.system('rm -rf ' + vis_spl)

    split(vis=msfile, outputvis=vis_spl, timerange=timeran, antenna=bl, field=field, scan=scan, spw=spw,
          uvrange=uvrange, timebin=timebin, datacolumn=datacolumn)
    ms.open(vis_spl, nomodify=False)
    if verbose:
        print('Regridding into a single spectral window...')
        # print('Reading data spw by spw')
    ms.cvel(outframe='LSRK', mode='frequency', interp='nearest')
    ms.selectinit(datadescid=0, reset=True)
    data = ms.getdata(['amplitude', 'time', 'axis_info'], ifraxis=True)
    ms.close()
    os.system('rm -rf ' + vis_spl)
    specamp = data['amplitude']
    (npol, nfreq, nbl, ntim) = specamp.shape
    if verbose:
        print('npol, nfreq, nbl, ntime:', data['amplitude'].shape)
    spec = np.swapaxes(specamp, 2, 1)
    freq = data['axis_info']['freq_axis']['chan_freq'].reshape(nfreq)
    tim = data['time']

    if domedian:
        if verbose:
            print('doing median of all the baselines')
        # mask zero values before median
        spec_masked = np.ma.masked_where(spec < 1e-9, spec)
        spec_med = np.ma.filled(np.ma.median(spec_masked, axis=1), fill_value=0.)
        nbl = 1
        ospec = spec_med.reshape((npol, nbl, nfreq, ntim))
    else:
        ospec = spec
    # Save the dynamic spectral data
    if savespec:
        if not specfile:
            specfile = msfile + '.dspec.npz'
        if os.path.exists(specfile):
            os.system('rm -rf ' + specfile)
        np.savez(specfile, spec=ospec, tim=tim, freq=freq,
                 timeran=timeran, spw=spw, bl=bl, uvrange=uvrange)
        if verbose:
            print('Median dynamic spectrum saved as: ' + specfile)

    return {'spec': ospec, 'tim': tim, 'freq': freq, 'timeran': timeran, 'spw': spw, 'bl': bl, 'uvrange': uvrange}


def plt_dspec(specdata, pol='I', dmin=None, dmax=None,
              timerange=None, freqrange=None, timestr=True,
              movie=False, framedur=60., dtframe=10.,
              goessav=None, goes_trange=None,
              savepng=True, savepdf=False):
    """
    timerange: format: ['2012/03/10/18:00:00','2012/03/10/19:00:00']
    freqrange: format: [1000.,1500.] in MHz
    movie: do a movie of dynamic spectrum?
    framedur: time range of each frame
    dtframe: time difference of consecutive frames
    goessav: provide an IDL save file from the sswidl GOES widget output
    goes_trange: plot only the specified time range for goes
    timestr: display time as strings on X-axis -- currently the times do not update themselves when zooming in
    """
    # Set up variables 
    import matplotlib.pyplot as plt
    import numpy
    from numpy import log10
    from astropy.time import Time
    if pol != 'RR' and pol != 'LL' and pol != 'RRLL' and pol != 'I' and pol != 'V' and pol != 'IV':
        print("Please enter 'RR', 'LL', 'RRLL', 'I', 'V', 'IV' for pol")
        return 0

    if type(specdata) is str:
        specdata = np.load(specdata)
        bl = specdata['bl'].item()
    try:
        (npol, nbl, nfreq, ntim) = specdata['spec'].shape
        spec = specdata['spec']
        tim = specdata['tim']
        tim_ = Time(tim / 3600. / 24., format='mjd')
        tim_plt = tim_.plot_date
        freq = specdata['freq']
        if not 'bl' in vars():
            bl = specdata['bl']
    except:
        print('format of specdata not recognized. Check your input')
        return -1

    if timerange:
        if type(timerange[0]) is str:
            timerange = [qa.convert(qa.quantity(t), 's')['value'] for t in timerange]
        tidx = np.where((tim >= timerange[0]) & (tim <= timerange[1]))[0]
    else:
        tidx = range(ntim)
    if freqrange:
        fidx = np.where((freq >= freqrange[0] * 1e6) & (freq <= freqrange[1] * 1e6))[0]
    else:
        fidx = range(nfreq)

    # setup plot parameters
    print('ploting dynamic spectrum...')
    spec_med = np.median(np.absolute(spec))
    # if not dmin:
    #    dmin = spec_med / 20.
    # if not dmax:
    #    dmax = spec_med * 5.
    # do the plot
    for b in range(nbl):
        if pol != 'RRLL' and pol != 'IV':
            if pol == 'RR':
                spec_plt = spec[0, b, :, :]
            elif pol == 'LL':
                spec_plt = spec[1, b, :, :]
            elif pol == 'I':
                spec_plt = (spec[0, b, :, :] + spec[1, b, :, :]) / 2.
            elif pol == 'V':
                spec_plt = (spec[0, b, :, :] - spec[1, b, :, :]) / 2.
            if movie:
                f = plt.figure(figsize=(16, 8), dpi=100)
                if goessav:
                    gs = gridspec.GridSpec(2, 1, height_ratios=[2, 1])
                    gs.update(left=0.06, right=0.97, top=0.95, bottom=0.06)
                    ax1 = f.add_subplot(gs[0])
                    ax2 = f.add_subplot(gs[1])
                    if os.path.exists(goessav):
                        goes = readsav(goessav)
                        # IDL anytim 0 sec correspond to 1979 Jan 01, convert to mjd time
                        anytimbase = qa.convert(qa.quantity('1979/01/01/00:00:00'), 's')['value']
                        mjdbase = goes['utbase'] + anytimbase
                        ts = goes['tarray'] + mjdbase
                        lc0 = goes['yclean'][0, :]
                        lc1 = goes['yclean'][1, :]
                else:
                    ax1 = f.add_subplot(211)
                tstart = tim[tidx[0]]
                tend = tim[tidx[-1]]
                tstartstr = qa.time(qa.quantity(tstart, 's'))[0]
                tendstr = qa.time(qa.quantity(tend, 's'))[0]
                nfrm = int((tend - tstart) / dtframe) + 1
                print('Movie mode set. ' + str(nfrm) + ' frames to plot from ' + tstartstr + ' to ' + tendstr)
                for i in range(nfrm):
                    if (i != 0) and (i % 10 == 0):
                        print(str(i) + ' frames done')
                    timeran = [tstart + i * dtframe, tstart + i * dtframe + framedur]
                    tidx1 = np.where((tim >= timeran[0]) & (tim <= timeran[1]))[0]
                    tim1 = tim_[tidx1]
                    freq1 = freq[fidx] / 1e9
                    spec_plt1 = spec_plt[fidx, :][:, tidx1]
                    ax1.pcolormesh(tim1.plot_date, freq1, spec_plt1, cmap='jet', vmin=dmin, vmax=dmax)
                    ax1.set_xlim(tim1[0].plot_date, tim1[-1].plot_date)
                    ax1.set_ylim(freq1[0], freq1[-1])
                    ax1.set_ylabel('Frequency (GHz)')
                    ax1.set_title('Dynamic spectrum @ bl ' + bl.split(';')[b] + ', pol ' + pol)
                    if timestr:
                        # date_format = mdates.DateFormatter('%H:%M:%S.%f')
                        # ax1.xaxis_date()
                        # ax1.xaxis.set_major_formatter(date_format)
                        locator = AutoDateLocator()
                        ax1.xaxis.set_major_locator(locator)
                        ax1.xaxis.set_major_formatter(AutoDateFormatter(locator))
                    ax1.set_autoscale_on(False)
                    if goessav:
                        if goes_trange:
                            if type(goes_trange[0]) is str:
                                goes_trange = [qa.convert(qa.quantity(t), 's')['value'] for t in goes_trange]
                                idx = np.where((ts >= goes_trange[0]) & (ts <= goes_trange[1]))[0]
                        else:
                            idx = range(len(ts))
                        ts_plt = ts[idx]
                        lc0_plt = lc0[idx]
                        utbase = qa.convert(qa.quantity('0001/01/01/00:00:00'), 'd')['value'] + 1
                        ts_plt_d = ts_plt / 3600. / 24. - utbase
                        ax2.plot_date(ts_plt_d, lc0_plt, 'b-')
                        ax2.axvspan(tim1[0].mjd - utbase, tim1[-1].mjd - utbase, color='red',
                                    alpha=0.5)
                        ax2.set_yscale('log')
                        ax2.set_title('GOES 1-8 A')

                    tstartstr_ = tim1[0].datetime.strftime('%Y-%m-%dT%H%M%S.%f')[:-3]
                    tendstr_ = tim1[1].datetime.strftime('%H%M%S.%f')[:-3]
                    timstr = tstartstr_ + '-' + tendstr_
                    figfile = 'dspec_t' + timstr + '.png'
                    if not os.path.isdir('dspec'):
                        os.makedirs('dspec')
                    f.savefig('dspec/' + figfile)
                    plt.cla()
            else:
                f = plt.figure(figsize=(8, 4), dpi=100)
                ax = f.add_subplot(111)
                freqghz = freq / 1e9
                ax.pcolormesh(tim_plt, freqghz, spec_plt, cmap='jet', vmin=dmin, vmax=dmax)
                ax.set_xlim(tim_plt[tidx[0]], tim_plt[tidx[-1]])
                ax.set_ylim(freqghz[fidx[0]], freqghz[fidx[-1]])
                try:
                    from sunpy import lightcurve
                    from sunpy.time import TimeRange, parse_time
                    t1 = tim_[tidx[0]]
                    t2 = tim_[tidx[-1]]
                    tr = TimeRange(t1.iso, t2.iso)
                    goes = lightcurve.GOESLightCurve.create(tr)
                    goes.data['xrsb'] = 2 * (np.log10(goes.data['xrsb'])) + 26
                    xx = [str(ll) for ll in np.array(goes.data.index)]
                    yy = np.array(goes.data['xrsb'])
                    ax.plot(Time(xx).mjd * 24 * 3600, yy, c='yellow')
                    rightaxis_label_time = Time(xx[-1]).mjd * 24 * 3600
                    ax.text(rightaxis_label_time, 9.6, 'A', fontsize='15')
                    ax.text(rightaxis_label_time, 11.6, 'B', fontsize='15')
                    ax.text(rightaxis_label_time, 13.6, 'C', fontsize='15')
                    ax.text(rightaxis_label_time, 15.6, 'M', fontsize='15')
                    ax.text(rightaxis_label_time, 17.6, 'X', fontsize='15')
                except:
                    pass

                def format_coord(x, y):
                    col = np.argmin(np.absolute(tim_plt - x))
                    row = np.argmin(np.absolute(freqghz - y))
                    if col >= 0 and col < ntim and row >= 0 and row < nfreq:
                        timstr = tim_[col].isot
                        flux = spec_plt[row, col]
                        return 'time {0} = {1}, freq = {2:.3f} GHz, flux = {3:.2f} Jy'.format(col, timstr, y, flux)
                    else:
                        return 'x = {0}, y = {1:.3f}'.format(x, y)

                ax.format_coord = format_coord
                ax.set_ylabel('Frequency (GHz)')
                if bl:
                    ax.set_title('Dynamic spectrum @ bl ' + bl.split(';')[b] + ', pol ' + pol)
                else:
                    ax.set_title('Medium dynamic spectrum')
                if timestr:
                    # date_format = mdates.DateFormatter('%H:%M:%S.%f')
                    # ax.xaxis_date()
                    # ax.xaxis.set_major_formatter(date_format)
                    locator = AutoDateLocator()
                    ax.xaxis.set_major_locator(locator)
                    ax.xaxis.set_major_formatter(AutoDateFormatter(locator))
                ax.set_autoscale_on(False)

        else:
            f = plt.figure(figsize=(8, 6), dpi=100)
            R_plot = np.absolute(spec[0, b, :, :])
            L_plot = np.absolute(spec[1, b, :, :])
            I_plot = (R_plot + L_plot) / 2.
            V_plot = (R_plot - L_plot) / 2.
            if pol == 'RRLL':
                spec_plt_1 = R_plot
                spec_plt_2 = L_plot
                polstr = ['RR', 'LL']
            if pol == 'IV':
                spec_plt_1 = I_plot
                spec_plt_2 = V_plot
                polstr = ['I', 'V']

            ax1 = f.add_subplot(211)
            freqghz = freq / 1e9
            ax1.pcolormesh(tim_plt, freqghz, spec_plt_1, cmap='jet', vmin=dmin, vmax=dmax)
            ax1.set_xlim(tim_plt[tidx[0]], tim_plt[tidx[-1]])
            ax1.set_ylim(freqghz[fidx[0]], freqghz[fidx[-1]])

            def format_coord(x, y):
                col = np.argmin(np.absolute(tim_plt - x))
                row = np.argmin(np.absolute(freqghz - y))
                if col >= 0 and col < ntim and row >= 0 and row < nfreq:
                    timstr = tim_[col].isot
                    flux = spec_plt[row, col]
                    return 'time {0} = {1}, freq = {2:.3f} GHz, flux = {3:.2f} Jy'.format(col, timstr, y, flux)
                else:
                    return 'x = {0}, y = {1:.3f}'.format(x, y)

            ax1.format_coord = format_coord
            ax1.set_ylabel('Frequency (GHz)')
            if timestr:
                # date_format = mdates.DateFormatter('%H:%M:%S.%f')
                # ax1.xaxis_date()
                # ax1.xaxis.set_major_formatter(date_format)
                locator = AutoDateLocator()
                ax1.xaxis.set_major_locator(locator)
                ax1.xaxis.set_major_formatter(AutoDateFormatter(locator))
            ax1.set_title('Dynamic spectrum @ bl ' + bl.split(';')[b] + ', pol ' + polstr[0])
            ax1.set_autoscale_on(False)
            ax2 = f.add_subplot(212)
            ax2.pcolormesh(tim_plt, freqghz, spec_plt_2, cmap='jet', vmin=dmin, vmax=dmax)
            ax2.set_xlim(tim_plt[tidx[0]], tim_plt[tidx[-1]])
            ax2.set_ylim(freqghz[fidx[0]], freqghz[fidx[-1]])
            if timestr:
                # date_format = mdates.DateFormatter('%H:%M:%S.%f')
                # ax2.xaxis_date()
                # ax2.xaxis.set_major_formatter(date_format)
                locator = AutoDateLocator()
                ax2.xaxis.set_major_locator(locator)
                ax2.xaxis.set_major_formatter(AutoDateFormatter(locator))

            def format_coord(x, y):
                col = np.argmin(np.absolute(tim_plt - x))
                row = np.argmin(np.absolute(freqghz - y))
                if col >= 0 and col < ntim and row >= 0 and row < nfreq:
                    timstr = tim_[col].isot
                    flux = spec_plt[row, col]
                    return 'time {0} = {1}, freq = {2:.3f} GHz, flux = {3:.2f} Jy'.format(col, timstr, y, flux)
                else:
                    return 'x = {0}, y = {1:.3f}'.format(x, y)

            ax2.format_coord = format_coord
            ax2.set_ylabel('Frequency (GHz)')
            ax2.set_title('Dynamic spectrum @ bl ' + bl.split(';')[b] + ', pol ' + polstr[1])
            ax2.set_autoscale_on(False)


def wrt_dspec(specfile=None, specdat=None):
    try:
        specfile
    except NameError:
        print('No input centfile specified for reading. Aborting...')
    if not specdat:
        print('Output file name is not specified, use the default convention')
        specdat = specfile.replace('npz', 'dat')
    specdata = np.load(specfile)
    spec = np.copy(specdata['spec'][:, :, :, :])
    npl, nbl, nf, nt = spec.shape
    print('Dimension of the data cube -- # of pol, # of baseline, # of frequency, # of time:')
    print(npl, nbl, nf, nt)
    nele = npl * nbl * nf * nt
    # need to transpose to nf, nt, npl, nbl
    spec = np.transpose(spec, (2, 3, 0, 1))
    # Turn fmx into a 1d array for writing
    spec = spec.flatten()
    # Pack the data into a byte buffer.  Note the asterisks in the next three lines!
    # If your data includes integers or other data types, use 'i' or other instead of 'f'
    buf = struct.pack(str(nf) + 'f', *specdata['freq'])
    buf += struct.pack(str(nt) + 'd', *specdata['tim'])
    buf += struct.pack(str(nele) + 'f', *spec)
    with open(specdat, 'wb') as f:
        f.write(buf)
    f.close()



#!/common/casa/casa-release-5.4.1-31.el6/bin/casa
from suncasa.eovsa import eovsa_pipeline as ep
from eovsapy.html_movie import html_movie
import sys
syspath = sys.path
sys.path = [s for s in syspath if '.local' not in s]

# Set to run 5 days earlier than current date
mjdnow = ep.Time.now().mjd
t = ep.Time(mjdnow - 6, format='mjd')
# Uncomment below and set date to run for a given date
# t = ep.Time('2019-09-17 20:00')
date = t.iso[:10]
# ep.qlook_image_pipeline(date, ncpu=1, synoptic=True)
ep.qlook_image_pipeline(date, ncpu=1)
html_movie(t, imgprefix='eovsa_qlimg_')
# html_movie(t, imgprefix='eovsa_qlimg_', synoptic=True)



import matplotlib

matplotlib.use('Agg')
import os
import shutil
import numpy as np
from eovsapy import refcal_anal as ra
from eovsapy.util import Time
from taskinit import casalog, tb, ms
from gencal_cli import gencal_cli as gencal
from clearcal_cli import clearcal_cli as clearcal
from applycal_cli import applycal_cli as applycal
from clean_cli import clean_cli as clean
from split_cli import split_cli as split
from bandpass_cli import bandpass_cli as bandpass
from flagdata_cli import flagdata_cli as flagdata
from eovsapy import cal_header as ch
from eovsapy import stateframe as stf
from eovsapy import dbutil as db
from eovsapy import pipeline_cal as pc


def calibeovsa(vis=None, caltype=None, caltbdir='', interp=None, docalib=True, doflag=True, flagant='13~15',
               doimage=False, imagedir=None, antenna=None, timerange=None, spw=None, stokes=None, dosplit=False,
               outputvis=None, doconcat=False, concatvis=None, keep_orig_ms=True):
    '''

    :param vis: EOVSA visibility dataset(s) to be calibrated 
    :param caltype:
    :param interp:
    :param docalib:
    :param qlookimage:
    :param flagant:
    :param stokes:
    :param doconcat:
    :return:
    '''

    if type(vis) == str:
        vis = [vis]

    for idx, f in enumerate(vis):
        if f[-1] == '/':
            vis[idx] = f[:-1]
        vis[idx] = str(vis[idx])

    # check if the calibration table directory is defined
    # pipeline should always use "caltbdir = /data1/eovsa/caltable/"
    if not caltbdir:
        print('Task calibeovsa')
        print('Path for generating calibration tables not defined')
        print('Use current path')
        caltbdir = './'

    for msfile in vis:
        casalog.origin('calibeovsa')
        if not caltype:
            casalog.post("Caltype not provided. Perform reference phase calibration and daily phase calibration.")
            caltype = ['refpha', 'phacal']  ## use this line after the phacal is applied  # caltype = ['refcal']
        if not os.path.exists(msfile):
            casalog.post("Input visibility does not exist. Aborting...")
            continue
        if msfile.endswith('/'):
            msfile = msfile[:-1]
        if not msfile[-3:] in ['.ms', '.MS']:
            casalog.post("Invalid visibility. Please provide a proper visibility file ending with .ms")
        # if not caltable:
        #    caltable=[os.path.basename(vis).replace('.ms','.'+c) for c in caltype]

        # get band information
        tb.open(msfile + '/SPECTRAL_WINDOW')
        nspw = tb.nrows()
        bdname = tb.getcol('NAME')
        bd_nchan = tb.getcol('NUM_CHAN')
        bd = [int(b[4:]) - 1 for b in bdname]
        # nchans = tb.getcol('NUM_CHAN')
        # reffreqs = tb.getcol('REF_FREQUENCY')
        # cenfreqs = np.zeros((nspw))
        tb.close()
        tb.open(msfile + '/ANTENNA')
        nant = tb.nrows()
        antname = tb.getcol('NAME')
        antlist = [str(ll) for ll in range(len(antname) - 1)]
        antennas = ','.join(antlist)
        tb.close()

        # get time stamp, use the beginning of the file
        tb.open(msfile + '/OBSERVATION')
        trs = {'BegTime': [], 'EndTime': []}
        for ll in range(tb.nrows()):
            tim0, tim1 = Time(tb.getcell('TIME_RANGE', ll) / 24 / 3600, format='mjd')
            trs['BegTime'].append(tim0)
            trs['EndTime'].append(tim1)
        tb.close()
        trs['BegTime'] = Time(trs['BegTime'])
        trs['EndTime'] = Time(trs['EndTime'])
        btime = np.min(trs['BegTime'])
        etime = np.max(trs['EndTime'])
        # ms.open(vis)
        # summary = ms.summary()
        # ms.close()
        # btime = Time(summary['BeginTime'], format='mjd')
        # etime = Time(summary['EndTime'], format='mjd')
        ## stop using ms.summary to avoid conflicts with importeovsa
        t_mid = Time((btime.mjd + etime.mjd) / 2., format='mjd')
        print("This scan observed from {} to {} UTC".format(btime.iso, etime.iso))
        gaintables = []

        if ('refpha' in caltype) or ('refamp' in caltype) or ('refcal' in caltype):
            refcal = ra.sql2refcalX(btime)
            pha = refcal['pha']  # shape is 15 (nant) x 2 (npol) x 34 (nband)
            pha[np.where(refcal['flag'] == 1)] = 0.
            amp = refcal['amp']
            amp[np.where(refcal['flag'] == 1)] = 1.
            t_ref = refcal['timestamp']
            # find the start and end time of the local day when refcal is registered
            try:
                dhr = t_ref.LocalTime.utcoffset().total_seconds() / 60. / 60.
            except:
                dhr = -7.
            bt = Time(np.fix(t_ref.mjd + dhr / 24.) - dhr / 24., format='mjd')
            et = Time(bt.mjd + 1., format='mjd')
            (yr, mon, day) = (bt.datetime.year, bt.datetime.month, bt.datetime.day)
            dirname = caltbdir + str(yr) + str(mon).zfill(2) + '/'
            if not os.path.exists(dirname):
                os.mkdir(dirname)
            # check if there is any ROACH reboot between the reference calibration found and the current data
            t_rbts = db.get_reboot(Time([t_ref, btime]))
            if not t_rbts:
                casalog.post("Reference calibration is derived from observation at " + t_ref.iso)
                print("Reference calibration is derived from observation at " + t_ref.iso)
            else:
                casalog.post(
                    "Oh crap! Roach reboot detected between the reference calibration time " + t_ref.iso + ' and the current observation at ' + btime.iso)
                casalog.post("Aborting...")
                print(
                    "Oh crap! Roach reboot detected between the reference calibration time " + t_ref.iso + ' and the current observation at ' + btime.iso)
                print("Aborting...")

            para_pha = []
            para_amp = []
            calpha = np.zeros((nspw, nant - 1, 2))
            calamp = np.zeros((nspw, nant - 1, 2))
            for s in range(nspw):
                for n in range(nant - 1):
                    for p in range(2):
                        calpha[s, n, p] = pha[n, p, bd[s]]
                        calamp[s, n, p] = amp[n, p, bd[s]]
                        para_pha.append(np.degrees(pha[n, p, bd[s]]))
                        para_amp.append(amp[n, p, bd[s]])

        if 'fluxcal' in caltype:
            calfac = pc.get_calfac(Time(t_mid.iso.split(' ')[0] + 'T23:59:59'))
            t_bp = Time(calfac['timestamp'], format='lv')
            if int(t_mid.mjd) == int(t_bp.mjd):
                accalfac = calfac['accalfac']  # (ant x pol x freq)
                # tpcalfac = calfac['tpcalfac']  # (ant x pol x freq)
                caltb_autoamp = dirname + t_bp.isot[:-4].replace(':', '').replace('-', '') + '.bandpass'
                if not os.path.exists(caltb_autoamp):
                    bandpass(vis=msfile, caltable=caltb_autoamp, solint='inf', refant='eo01', minblperant=0, minsnr=0,
                             bandtype='B', docallib=False)
                    tb.open(caltb_autoamp, nomodify=False)  # (ant x spw)
                    bd_chanidx = np.hstack([[0], bd_nchan.cumsum()])
                    for ll in range(nspw):
                        antfac = np.sqrt(accalfac[:, :, bd_chanidx[ll]:bd_chanidx[ll + 1]])
                        # # antfac *= tpcalfac[:, :,bd_chanidx[ll]:bd_chanidx[ll + 1]]
                        antfac = np.moveaxis(antfac, 0, 2)
                        cparam = np.zeros((2, bd_nchan[ll], nant))
                        cparam[:, :, :-3] = 1.0 / antfac
                        tb.putcol('CPARAM', cparam + 0j, ll * nant, nant)
                        paramerr = tb.getcol('PARAMERR', ll * nant, nant)
                        paramerr = paramerr * 0
                        tb.putcol('PARAMERR', paramerr, ll * nant, nant)
                        bpflag = tb.getcol('FLAG', ll * nant, nant)
                        bpant1 = tb.getcol('ANTENNA1', ll * nant, nant)
                        bpflagidx, = np.where(bpant1 >= 13)
                        bpflag[:] = False
                        bpflag[:, :, bpflagidx] = True
                        tb.putcol('FLAG', bpflag, ll * nant, nant)
                        bpsnr = tb.getcol('SNR', ll * nant, nant)
                        bpsnr[:] = 100.0
                        bpsnr[:, :, bpflagidx] = 0.0
                        tb.putcol('SNR', bpsnr, ll * nant, nant)
                    tb.close()
                    msg_prompt = "Scaling calibration is derived for {}.".format(msfile)
                    casalog.post(msg_prompt)
                    print(msg_prompt)
                gaintables.append(caltb_autoamp)
            else:
                msg_prompt = "Caution: No TPCAL is available on {}. No scaling calibration is derived for {}.".format(
                    t_mid.datetime.strftime('%b %d, %Y'), msfile)
                casalog.post(msg_prompt)
                print(msg_prompt)

        if ('refpha' in caltype) or ('refcal' in caltype):
            # caltb_pha = os.path.basename(vis).replace('.ms', '.refpha')
            # check if the calibration table already exists
            caltb_pha = dirname + t_ref.isot[:-4].replace(':', '').replace('-', '') + '.refpha'
            if not os.path.exists(caltb_pha):
                gencal(vis=msfile, caltable=caltb_pha, caltype='ph', antenna=antennas, pol='X,Y',
                       spw='0~' + str(nspw - 1), parameter=para_pha)
                tb.open(caltb_pha, nomodify=False)
                phaflag_ = refcal['flag'][:, :, np.array(bd)]
                phaflag_new = np.full((nant, 2, nspw), True, dtype=np.bool)
                phaflag_new[:-1, ...] = phaflag_
                phaflag_new = np.moveaxis(phaflag_new, 0, 2).reshape(2, 1, nant*nspw)
                tb.putcol('FLAG', phaflag_new)
                tb.close()

                # tb.open(caltb_pha, nomodify=False)
                # phaparam = np.angle(tb.getcol('CPARAM'),deg=True)
                # phaparam_ = np.degrees(refcal['pha'][:,:,np.array(bd)])
                # phaparam2 = np.zeros((nant, 2, nspw))
                # phaparam2[:-1,...] = phaparam_
                # # phaparam2 = phaparam2.swapaxes(0,1).reshape(2,1,nant*nspw)
                # phaparam2 = np.moveaxis(phaparam2,0,2).reshape(2,1,nant*nspw)
                # tb.close()

            gaintables.append(caltb_pha)
        if ('refamp' in caltype) or ('refcal' in caltype):
            # caltb_amp = os.path.basename(vis).replace('.ms', '.refamp')
            caltb_amp = dirname + t_ref.isot[:-4].replace(':', '').replace('-', '') + '.refamp'
            if not os.path.exists(caltb_amp):
                gencal(vis=msfile, caltable=caltb_amp, caltype='amp', antenna=antennas, pol='X,Y',
                       spw='0~' + str(nspw - 1), parameter=para_amp)
                tb.open(caltb_amp, nomodify=False)
                ampflag_ = refcal['flag'][:, :, np.array(bd)]
                ampflag_new = np.full((nant, 2, nspw), True, dtype=np.bool)
                ampflag_new[:-1, ...] = ampflag_
                ampflag_new = np.moveaxis(ampflag_new, 0, 2).reshape(2, 1, nant*nspw)
                tb.putcol('FLAG', ampflag_new)
                tb.close()
            gaintables.append(caltb_amp)

        # calibration for the change of delay center between refcal time and beginning of scan -- hopefully none!
        xml, buf = ch.read_calX(4, t=[t_ref, btime], verbose=False)
        if buf:
            dly_t2 = Time(stf.extract(buf[0], xml['Timestamp']), format='lv')
            dlycen_ns2 = stf.extract(buf[0], xml['Delaycen_ns'])[:nant - 1]
            xml, buf = ch.read_calX(4, t=t_ref)
            dly_t1 = Time(stf.extract(buf, xml['Timestamp']), format='lv')
            dlycen_ns1 = stf.extract(buf, xml['Delaycen_ns'])[:nant - 1]
            dlycen_ns_diff = dlycen_ns2 - dlycen_ns1
            for n in range(2):
                dlycen_ns_diff[:, n] -= dlycen_ns_diff[0, n]
            print('Multi-band delay is derived from delay center difference at {} & {}'.format(dly_t1.iso, dly_t2.iso))
            # print('=====Delays relative to Ant 14=====')
            # for i, dl in enumerate(dlacen_ns_diff[:, 0] - dlacen_ns_diff[13, 0]):
            #     ant = antlist[i]
            #     print 'Ant eo{0:02d}: x {1:.2f} ns & y {2:.2f} ns'.format(int(ant) + 1, dl
            #           dlacen_ns_diff[i, 1] - dlacen_ns_diff[13, 1])
            # caltb_mbd0 = os.path.basename(vis).replace('.ms', '.mbd0')
            caltb_dlycen = dirname + dly_t2.isot[:-4].replace(':', '').replace('-', '') + '.dlycen'
            if not os.path.exists(caltb_dlycen):
                gencal(vis=msfile, caltable=caltb_dlycen, caltype='mbd', pol='X,Y', antenna=antennas,
                       parameter=dlycen_ns_diff.flatten().tolist())
            gaintables.append(caltb_dlycen)

        if 'phacal' in caltype:
            phacals = np.array(ra.sql2phacalX([bt, et], neat=True, verbose=False))
            if not phacals.any() or len(phacals) == 0:
                print("Found no phacal records in SQL database, will skip phase calibration")
            else:
                # first generate all phacal calibration tables if not already exist
                t_phas = Time([phacal['t_pha'] for phacal in phacals])
                # sort the array in ascending order by t_pha
                sinds = t_phas.mjd.argsort()
                t_phas = t_phas[sinds]
                phacals = phacals[sinds]
                caltbs_phambd = []
                for i, phacal in enumerate(phacals):
                    # filter out phase cals with reference time stamp >30 min away from the provided refcal time
                    if (phacal['t_ref'].jd - refcal['timestamp'].jd) > 30. / 1440.:
                        del phacals[i]
                        del t_phas[i]
                        continue
                    else:
                        t_pha = phacal['t_pha']
                        phambd_ns = phacal['pslope']
                        for n in range(2): phambd_ns[:, n] -= phambd_ns[0, n]
                        # set all flagged values to be zero
                        phambd_ns[np.where(phacal['flag'] == 1)] = 0.
                        caltb_phambd = dirname + t_pha.isot[:-4].replace(':', '').replace('-', '') + '.phambd'
                        caltbs_phambd.append(caltb_phambd)
                        if not os.path.exists(caltb_phambd):
                            gencal(vis=msfile, caltable=caltb_phambd, caltype='mbd', pol='X,Y', antenna=antennas,
                                   parameter=phambd_ns.flatten().tolist())

                # now decides which table to apply depending on the interpolation method ("neatest" or "linear")
                if interp == 'nearest':
                    tbind = np.argmin(np.abs(t_phas.mjd - t_mid.mjd))
                    dt = np.min(np.abs(t_phas.mjd - t_mid.mjd)) * 24.
                    print("Selected nearest phase calibration table at " + t_phas[tbind].iso)
                    gaintables.append(caltbs_phambd[tbind])
                if interp == 'linear':
                    # bphacal = ra.sql2phacalX(btime)
                    # ephacal = ra.sql2phacalX(etime,reverse=True)
                    bt_ind, = np.where(t_phas.mjd < btime.mjd)
                    et_ind, = np.where(t_phas.mjd > etime.mjd)
                    if len(bt_ind) == 0 and len(et_ind) == 0:
                        print("No phacal found before or after the ms data within the day of observation")
                        print("Skipping daily phase calibration")
                    elif len(bt_ind) > 0 and len(et_ind) == 0:
                        gaintables.append(caltbs_phambd[bt_ind[-1]])
                    elif len(bt_ind) == 0 and len(et_ind) > 0:
                        gaintables.append(caltbs_phambd[et_ind[0]])
                    elif len(bt_ind) > 0 and len(et_ind) > 0:
                        bphacal = phacals[bt_ind[-1]]
                        ephacal = phacals[et_ind[0]]
                        # generate a new table interpolating between two daily phase calibrations
                        t_pha_mean = Time(np.mean([bphacal['t_pha'].mjd, ephacal['t_pha'].mjd]), format='mjd')
                        phambd_ns = (bphacal['pslope'] + ephacal['pslope']) / 2.
                        for n in range(2): phambd_ns[:, n] -= phambd_ns[0, n]
                        # set all flagged values to be zero
                        phambd_ns[np.where(bphacal['flag'] == 1)] = 0.
                        phambd_ns[np.where(ephacal['flag'] == 1)] = 0.
                        caltb_phambd_interp = dirname + t_pha_mean.isot[:-4].replace(':', '').replace('-',
                                                                                                      '') + '.phambd'
                        if not os.path.exists(caltb_phambd_interp):
                            gencal(vis=msfile, caltable=caltb_phambd_interp, caltype='mbd', pol='X,Y', antenna=antennas,
                                   parameter=phambd_ns.flatten().tolist())
                        print("Using phase calibration table interpolated between records at " + bphacal[
                            't_pha'].iso + ' and ' + ephacal['t_pha'].iso)
                        gaintables.append(caltb_phambd_interp)

        if docalib:
            clearcal(msfile)
            applycal(vis=msfile, gaintable=gaintables, applymode='calflag', calwt=False)
            # delete the interpolated phase calibration table
            try:
                caltb_phambd_interp
            except:
                pass
            else:
                if os.path.exists(caltb_phambd_interp):
                    shutil.rmtree(caltb_phambd_interp)
        if doflag:
            # flag zeros and NaNs
            flagdata(vis=msfile, mode='clip', clipzeros=True)
            if flagant:
                try:
                    flagdata(vis=msfile, antenna=flagant)
                except:
                    print("Something wrong with flagant. Abort...")

        if doimage:
            from matplotlib import pyplot as plt
            from suncasa.utils import helioimage2fits as hf
            from sunpy import map as smap

            if not antenna:
                antenna = '0~12'
            if not stokes:
                stokes = 'XX'
            if not timerange:
                timerange = ''
            if not spw:
                spw = '1~3'
            if not imagedir:
                imagedir = '.'
            # (yr, mon, day) = (bt.datetime.year, bt.datetime.month, bt.datetime.day)
            # dirname = imagedir + str(yr) + '/' + str(mon).zfill(2) + '/' + str(day).zfill(2) + '/'
            # if not os.path.exists(dirname):
            #    os.makedirs(dirname)
            bds = [spw]
            nbd = len(bds)
            imgs = []
            for bd in bds:
                if '~' in bd:
                    bdstr = bd.replace('~', '-')
                else:
                    bdstr = str(bd).zfill(2)
                imname = imagedir + '/' + os.path.basename(msfile).replace('.ms', '.bd' + bdstr)
                print('Cleaning image: ' + imname)
                try:
                    clean(vis=msfile, imagename=imname, antenna=antenna, spw=bd, timerange=timerange, imsize=[512],
                          cell=['5.0arcsec'], stokes=stokes,
                          niter=500)
                except:
                    print('clean not successfull for band ' + str(bd))
                else:
                    imgs.append(imname + '.image')
                junks = ['.flux', '.mask', '.model', '.psf', '.residual']
                for junk in junks:
                    if os.path.exists(imname + junk):
                        shutil.rmtree(imname + junk)

            tranges = [btime.iso + '~' + etime.iso] * nbd
            fitsfiles = [img.replace('.image', '.fits') for img in imgs]
            hf.imreg(vis=msfile, timerange=tranges, imagefile=imgs, fitsfile=fitsfiles, usephacenter=False)
            plt.figure(figsize=(6, 6))
            for i, fitsfile in enumerate(fitsfiles):
                plt.subplot(1, nbd, i + 1)
                eomap = smap.Map(fitsfile)
                sz = eomap.data.shape
                if len(sz) == 4:
                    eomap.data = eomap.data.reshape((sz[2], sz[3]))
                eomap.plot_settings['cmap'] = plt.get_cmap('jet')
                eomap.plot()
                eomap.draw_limb()
                # the next line would cause trouble in higher versions of SunPy, as it requires WCS
                # eomap.draw_grid()

            plt.show()

    if dosplit:
        if not doconcat:
            if not outputvis:
                outputvis = [vis[n].split('.')[0] + '.corrected.ms' for n in range(len(vis))]
            for n in range(len(vis)):
                split(vis=vis[n], outputvis=outputvis[n], datacolumn='corrected')
                if not keep_orig_ms:
                    os.system('rm -rf {}'.format(vis[n]))
    else:
        outputvis = vis

    if doconcat:
        # doconcat imply dosplit
        from suncasa.tasks import concateovsa_cli as ce
        # from suncasa.eovsa import concateovsa as ce
        if not concatvis:
            msoutdir = os.path.dirname(vis[0])
            if len(vis) == 1:
                vis0 = os.path.basename(vis[0])
                concatvis = os.path.join(msoutdir, vis0.split('.')[0] + '.corrected.ms')
            if len(vis) > 1:
                visb = os.path.basename(vis[0])
                vise = os.path.basename(vis[-1])
                concatvis = os.path.join(msoutdir, visb.split('.')[0] + '-' + vise.split('.')[0][3:] + '.corrected.ms')
        if len(vis) == 1:
            split(vis=vis[0], outputvis=concatvis, datacolumn='corrected')
        if len(vis) > 1:
            ce.concateovsa(vis, concatvis, datacolumn='corrected', keep_orig_ms=keep_orig_ms, cols2rm="model,corrected")
        return concatvis
    else:
        return outputvis



import numpy as np
import numpy.ma as ma
import shutil
import os, struct
import bisect
from math import *
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import matplotlib.colors as colors
import matplotlib.gridspec as gs
from fnmatch import fnmatch
from time import sleep
from time import time
from taskinit import casalog
import multiprocessing as mprocs
from suncasa.utils import DButil


def imfit_iter(imgfiles, doreg, tims, msinfofile, ephem, box, region, chans, stokes, mask, includepix, excludepix,
               residual, model, estimates, logfile, append, newestimates, complist,
               overwrite, dooff, offset, fixoffset, stretch, rms, noisefwhm, summary,
               imidx):
    from taskinit import iatool,rgtool
    import pdb
    try:
        from astropy.io import fits as pyfits
    except:
        try:
            import pyfits
        except ImportError:
            raise ImportError('Neither astropy nor pyfits exists in this CASA installation')
    img = imgfiles[imidx]

    if doreg:
        # check if ephemfile and msinfofile exist
        if not ephem:
            print("ephemeris info does not exist!")
            return
        if not tims:
            print("timestamp of the image does not exist!")
            return
        try:
            tim = tims[imidx]
            helio = vla_prep.ephem_to_helio(msinfo=msinfofile, ephem=ephem, reftime=tim)
            fitsfile = [img.replace('.image', '.fits')]
            vla_prep.imreg(imagefile=[img], fitsfile=fitsfile, helio=helio, toTb=False, scl100=True)
            img = img.replace('.image', '.fits')
        except:
            print 'Failure in vla_prep. Skipping this image file: ' + img

    myia = iatool()
    myrg = rgtool()
    try:
        if (not myia.open(img)):
            raise Exception, "Cannot create image analysis tool using " + img
        print('Processing image: ' + img)
        hdr = pyfits.getheader(img)
        pols = DButil.polsfromfitsheader(hdr)
        ndx, ndy, nchans, npols = myia.shape()
        results = {}
        for itpp in pols:
            results[itpp] = {}
        for pp, itpp in enumerate(pols):
            r = myrg.box(blc=[0, 0, 0, pp], trc=[ndx, ndy, nchans, pp])
            myiapol = myia.subimage(region=r, dropdeg=True)
            results[itpp] = myiapol.fitcomponents(
                box=box, region=region, chans=chans, stokes=stokes,
                mask=mask, includepix=includepix,
                excludepix=excludepix, residual=residual,
                model=model, estimates=estimates, logfile=logfile,
                append=append, newestimates=newestimates,
                complist=complist, overwrite=overwrite, dooff=dooff,
                offset=offset, fixoffset=fixoffset, stretch=stretch,
                rms=rms, noisefwhm=noisefwhm, summary=summary
            )
        # update timestamp

        timstr = hdr['date-obs']
        return [True, timstr, img, results]
    except Exception, instance:
        casalog.post(str('*** Error in imfit ***') + str(instance))
        # raise instance
        return [False, timstr, img, {}]
    finally:
        myia.done()


def pimfit(imagefiles, ncpu, doreg, timestamps, msinfofile, ephemfile, box, region, chans, stokes, mask, includepix,
           excludepix,
           residual, model, estimates, logfile, append, newestimates, complist,
           overwrite, dooff, offset, fixoffset, stretch, rms, noisefwhm, summary):
    from functools import partial
    import pdb
    # check if imagefiles is a single file or a list of files

    if isinstance(imagefiles, str):
        imagefiles = [imagefiles]
    if (not isinstance(imagefiles, list)):
        print 'input "imagefiles" is not a list. Abort...'

    if doreg:
        # check if ephemfile and msinfofile exist
        try:
            ephem = vla_prep.read_horizons(ephemfile=ephemfile)
        except ValueError:
            print("error in reading ephemeris file")
        if not os.path.isfile(msinfofile):
            print("msinfofile does not exist!")
        # check if timestamps exist 
        if (not isinstance(timestamps, list)):
            print('input "timestamps" is not a list. Abort...')
        if (len(timestamps) != len(imagefiles)):
            print('imagefiles and timestamps should have the same length! Abort...')
    else:
        ephem = None

    # check file existence
    imgfiles = []
    tims = []
    for i, img in enumerate(imagefiles):
        if os.path.exists(img):
            imgfiles.append(img)
            if doreg:
                tims.append(timestamps[i])
        else:
            casalog.post(img + 'does not exist. Skipping this one...')

    iterable = range(len(imgfiles))
    res = []

    if not (type(ncpu) is int):
        casalog.post('ncpu should be an integer')
        ncpu = 8

    # partition
    imfit_part = partial(imfit_iter, imgfiles, doreg, tims, msinfofile, ephem, box, region, chans, stokes, mask,
                         includepix, excludepix, \
                         residual, model, estimates, logfile, append, newestimates, complist, \
                         overwrite, dooff, offset, fixoffset, stretch, rms, noisefwhm, summary)

    # parallelization
    para = 1
    timelapse = 0
    t0 = time()
    if para:
        casalog.post('Perform imfit in parallel ...')
        pool = mprocs.Pool(ncpu)
        # res = pool.map_async(imfit_part, iterable)
        res = pool.map(imfit_part, iterable)
        pool.close()
        pool.join()
    else:
        for i in iterable:
            res.append(imfit_part(i))

    t1 = time()
    timelapse = t1 - t0
    print 'It took %f secs to complete' % timelapse
    # repackage this into a single dictionary
    results = {'succeeded': [], 'timestamps': [], 'imagenames': [], 'outputs': []}
    for r in res:
        results['succeeded'].append(r[0])
        results['timestamps'].append(r[1])
        results['imagenames'].append(r[2])
        results['outputs'].append(r[3])

    return results



#!/usr/bin/env python
from setuptools import setup
import os
from itertools import chain

# from extension_helpers import get_extensions
from setuptools.config import read_configuration

################################################################################
# Programmatically generate some extras combos.
################################################################################
install_requires = read_configuration("setup.cfg")['options']['install_requires']
extras = read_configuration("setup.cfg")['options']['extras_require']

# Dev is everything
extras['dev'] = list(chain(*extras.values()))

# All is everything but tests and docs
exclude_keys = ("tests", "docs", "dev")
ex_extras = dict(filter(lambda i: i[0] not in exclude_keys, extras.items()))
# Concatenate all the values together for 'all'
# extras['all'] = list(set(list(chain.from_iterable(ex_extras.values()))))
extras['all'] = list(chain.from_iterable(ex_extras.values()))

setup(
    install_requires=install_requires,
    extras_require=extras,
    # use_scm_version={'write_to': os.path.join('suncasa', '_version.py')},
    # version='0.1.2.9.1' ## test
    version='1.0.0' ## official
    # ,
    # ext_modules=get_extensions(),
)



def multicolor_text(axes, x, y, textin, cmap=None,wratio=0.5, bbox={}, **kw):
    import matplotlib.pyplot as plt
    from matplotlib import transforms
    """
        Take a list of strings ``ls`` and colors ``lc`` and place them next to each
        other, with text ls[i] being shown in color lc[i].

        This example shows how to do both vertical and horizontal text, and will
        pass all keyword arguments to plt.text, so you can set the font size,
        family, etc.
        """
    fig = plt.gcf()
    t = axes.transAxes

    # horizontal version
    if cmap is None:
        cmap = plt.cm.RdYlBu

    textin = list(textin)
    ntextin = len(textin)
    for idx, s in enumerate(textin):
        # s_ = ['_']*ntextin
        # s_[idx] = s
        c = cmap(float(idx) / (ntextin - 1))
        # tx = axes.text(x, y, ''.join(s_), color=c, transform=t, **kw)
        tx = axes.text(x, y, s, color=c, transform=t, **kw)
        if bbox:
            tx.set_bbox(bbox)
        tx.draw(fig.canvas.get_renderer())
        ex = tx.get_window_extent()
        t = transforms.offset_copy(tx._transform,fig=fig, x=ex.width*wratio, units='points')


def align_marker(marker, halign='center', valign='middle', ):
    from matplotlib import markers
    from matplotlib.path import Path
    """
    create markers with specified alignment.

    Parameters
    ----------

    marker : a valid marker specification.
      See mpl.markers

    halign : string, float {'left', 'center', 'right'}
      Specifies the horizontal alignment of the marker. *float* values
      specify the alignment in units of the markersize/2 (0 is 'center',
      -1 is 'right', 1 is 'left').

    valign : string, float {'top', 'middle', 'bottom'}
      Specifies the vertical alignment of the marker. *float* values
      specify the alignment in units of the markersize/2 (0 is 'middle',
      -1 is 'top', 1 is 'bottom').

    Returns
    -------

    marker_array : numpy.ndarray
      A Nx2 array that specifies the marker path relative to the
      plot target point at (0, 0).

    Notes
    -----
    The mark_array can be passed directly to ax.plot and ax.scatter, e.g.::

        ax.plot(1, 1, marker=align_marker('>', 'left'))

    """

    if isinstance(halign, str):
        halign = {'right': -1.,
                  'middle': 0.,
                  'center': 0.,
                  'left': 1.,
                  }[halign]

    if isinstance(valign, str):
        valign = {'top': -1.,
                  'middle': 0.,
                  'center': 0.,
                  'bottom': 1.,
                  }[valign]

    # Define the base marker
    bm = markers.MarkerStyle(marker)

    # Get the marker path and apply the marker transform to get the
    # actual marker vertices (they should all be in a unit-square
    # centered at (0, 0))
    m_arr = bm.get_path().transformed(bm.get_transform()).vertices

    # Shift the marker vertices for the specified alignment.
    m_arr[:, 0] += halign / 2
    m_arr[:, 1] += valign / 2

    return Path(m_arr, bm.get_path().codes)


import matplotlib.pyplot as plt
import matplotlib.dates as dates
import matplotlib.gridspec as gridspec
import numpy as np
import os
import datetime
import jdutil
import pdb
import signalsmooth
import struct
from scipy.io.idl import readsav
from datetime import datetime
from taskinit import *


def get_dspec(vis=None, specfile=None, bl=None, spw=None, timeran=None):
    """
    Note: antennas specified in "bl" is antennas INDEX but not antenna NAME.
    REQUIRED INPUTS:
        vis: name of the measurement set file
        bl: baseline pair to use for the dynamic spectrum in CASA msselect format. e.g., bl='10&11'
    OPTIONAL INPUTS:
        specfile: name of the dynamic spectrum file
        spw: in CASA msselect format, e.g., '0~7:10~30', '0~3' (other formats not supported yet)
        timeran: in CASA msselect format, e.g., '18:00:00~18:30:00'
    EXAMPLE:
        import dspec
        dspec.get_dspec(mspath='./',msfile='yourcasadata.ms',spw='0~7',
                        bl='2&3',timeran='18:00:00~18:30:00')
    """
    # Open the ms and plot dynamic spectrum
    if not spw:
        spw = ''
    if not timeran:
        timeran = ''
    if not bl:
        bl = ''
    if not specfile:
        specfile = vis + '.spec.npz'
    print 'Splitting selected dynamic spectral data...'
    ms.open(vis, nomodify=True)
    vis_spl = './tmpms.splitted'
    ms.split(outputms=vis_spl, whichcol='DATA', time=timeran, spw=spw, baseline=bl)
    ms.close()
    ms.open(vis_spl, nomodify=False)
    print 'Regridding into a single spectral window...'
    ms.cvel(outframe='LSRK', mode='frequency', interp='nearest')
    ms.selectinit(datadescid=0, reset=True)
    specdata = ms.getdata(['amplitude', 'time', 'axis_info'], ifraxis=True)
    ms.close()
    os.system('rm -rf ' + vis_spl)
    npol = specdata['amplitude'].shape[0]
    nfreq = specdata['amplitude'].shape[1]
    nbl = specdata['amplitude'].shape[2]
    ntim = specdata['amplitude'].shape[3]
    print 'shape of the spectral data: ', specdata['amplitude'].shape
    print 'number of selected polarizations: ', npol
    print 'number of selected baselines: ', nbl
    print 'number of selected frequency channels: ', nfreq
    print 'number of selected time pixels: ', ntim
    spec = np.swapaxes(specdata['amplitude'], 2, 1)
    # CASA 4.3.1
    freq = specdata['axis_info']['freq_axis']['chan_freq'].reshape(nfreq)
    tim = specdata['time']
    # Save variables
    np.savez(specfile, spec=spec, tim=tim, freq=freq,
             timeran=timeran, bl=bl, spw=spw,
             npol=npol, nbl=nbl, nfreq=nfreq, ntim=ntim)
    print 'Dynamic spectrum saved as: ' + specfile


def get_med_dspec(mspath=None, msfile=None, specfile=None, \
                  smoothmode='base', tsmoothwidth=40, tsmoothtype='hanning', \
                  fsmoothwidth=4, fsmoothtype='hannning', \
                  fsmooth=1, timeran=None, chanran=None, dmin=0.5, dmax=2.0):
    # Open the ms and plot dynamic spectrum
    print 'retrieving spectral data...'
    ms.open(mspath + '/' + msfile)
    ms.selectinit(datadescid=0)
    if timeran and (type(timeran) == str):
        ms.msselect({'time': timeran})
    if chanran and (type(chanran) == str):
        [bchan, echan] = chanran.split('~')
        nchan = int(echan) - int(bchan) + 1
        ms.selectchannel(nchan, int(bchan), 1, 1)
    specdata = ms.getdata(['data', 'time', 'axis_info'], ifraxis=True)
    npol = specdata['data'].shape[0]
    nchan = specdata['data'].shape[1]
    nbl = specdata['data'].shape[2]
    ntime = specdata['data'].shape[3]
    print 'shape of the spectral data: ', specdata['data'].shape
    print 'number of time pixels: ', specdata['time'].shape[0]
    print 'number of frequency channels selected: ', specdata['axis_info']['freq_axis']['chan_freq'].shape[0]
    print 'number of baselines: ', specdata['axis_info']['ifr_axis']['baseline'].shape[0]
    # ratio over time-average
    if smoothmode == 'hipass':
        specsmooth = np.copy(specdata['data'])
        # time smooth
        for i in range(npol):
            for j in range(nchan):
                for k in range(nbl):
                    specsmooth[i, j, k, :] = signalsmooth.smooth(specdata['data'][i, j, k, :], tsmoothwidth,
                                                                 tsmoothtype)
        # frequency smooth
        for i in range(npol):
            for j in range(nbl):
                for k in range(ntime):
                    specsmooth[i, :, j, k] = signalsmooth.smooth(specsmooth[i, :, j, k], fsmoothwidth, fsmoothtype)
        specratio = specdata['data'] / specsmooth
    if smoothmode == 'base':
        specsmooth = np.mean(specdata['data'], axis=3)
        specratio = specdata['data'] / specsmooth[:, :, :, None]

    spec_med = np.median(specratio, axis=2)
    if fsmooth:
        for i in range(npol):
            for j in range(ntime):
                spec_med[i, :, j] = signalsmooth.smooth(spec_med[i, :, j], fsmoothwidth, fsmoothtype)

    tim = specdata['time']
    freq = specdata['axis_info']['freq_axis']['chan_freq'].reshape(nchan)
    tim0 = tim[0]
    tim0str = qa.time(qa.quantity(tim0, 's'), prec=8)[0]
    tim_ = tim - tim[0]
    freqghz = freq / 1e9
    f = plt.figure(figsize=(8, 8), dpi=100)
    ax1 = f.add_subplot(211)
    f.subplots_adjust(hspace=0.4)
    ax1.pcolormesh(tim_, freqghz, np.abs(spec_med[0, :, :]), cmap='jet', vmin=dmin, vmax=dmax)
    ax1.set_xlim([tim_[0], tim_[-1]])
    ax1.set_ylim([freqghz[0], freqghz[-1]])
    ax1.set_title('Median-Filtered Dynamic Spectrum (RCP)')
    ax1.set_xlabel('Time (seconds) since ' + tim0str)
    ax1.set_ylabel('Frequency (GHz)')
    ax2 = f.add_subplot(212)
    ax2.pcolormesh(tim_, freqghz, np.abs(spec_med[1, :, :]), cmap='jet', vmin=dmin, vmax=dmax)
    ax2.set_xlim([tim_[0], tim_[-1]])
    ax2.set_ylim([freqghz[0], freqghz[-1]])
    ax2.set_title('Median-Filtered Dynamic Spectrum (LCP)')
    ax2.set_xlabel('Time (seconds) since ' + tim0str)
    ax2.set_ylabel('Frequency (GHz)')
    if specfile:
        np.savez(mspath + '/' + specfile, spec_med=spec_med, tim=tim, freq=freq)


def plt_dspec(specfile=None, pol='I', dmin=None, dmax=None,
              timerange=None, freqrange=None, timestr=True,
              movie=False, framedur=60., dtframe=10.,
              goessav=None, goes_trange=None,
              savepng=True, savepdf=False):
    """
    timerange: format: ['2012/03/10/18:00:00','2012/03/10/19:00:00']
    freqrange: format: [1000.,1500.] in MHz
    movie: do a movie of dynamic spectrum?
    framedur: time range of each frame
    dtframe: time difference of consecutive frames
    goessav: provide an IDL save file from the sswidl GOES widget output
    goes_trange: plot only the specified time range for goes
    timestr: display time as strings on X-axis -- currently the times do not update themselves when zooming in
    """
    # Set up variables 
    if pol != 'RR' and pol != 'LL' and pol != 'RRLL' and pol != 'I' and pol != 'V' and pol != 'IV':
        print "Please enter 'RR', 'LL', 'RRLL', 'I', 'V', 'IV' for pol"
        return 0

    specdata = np.load(specfile)
    spec = specdata['spec']
    tim = specdata['tim']
    freq = specdata['freq']
    bl = specdata['bl'].item()
    #npol = specdata['npol']
    #nbl = specdata['nbl']
    (npol, nbl, nfreq, ntim)=spec.shape

    if timerange:
        if type(timerange[0]) is str:
            timerange = [qa.convert(qa.quantity(t), 's')['value'] for t in timerange]
        tidx = np.where((tim >= timerange[0]) & (tim <= timerange[1]))[0]
    else:
        tidx = range(ntim)
    if freqrange:
        fidx = np.where((freq >= freqrange[0] * 1e6) & (freq <= freqrange[1] * 1e6))[0]
    else:
        fidx = range(nfreq)

    # setup plot parameters
    print 'ploting dynamic spectrum...'
    spec_med = np.median(np.absolute(spec))
    if not dmin:
        dmin = spec_med / 20.
    if not dmax:
        dmax = spec_med * 5.
    # do the plot
    for b in range(nbl):
        if pol != 'RRLL' and pol != 'IV':
            if pol == 'RR':
                spec_plt = spec[0, b, :, :]
            elif pol == 'LL':
                spec_plt = spec[1, b, :, :]
            elif pol == 'I':
                spec_plt = (spec[0, b, :, :] + spec[1, b, :, :]) / 2.
            elif pol == 'V':
                spec_plt = (spec[0, b, :, :] - spec[1, b, :, :]) / 2.
            if movie:
                f = plt.figure(figsize=(16, 8), dpi=100)
                if goessav:
                    gs = gridspec.GridSpec(2, 1, height_ratios=[2, 1])
                    gs.update(left=0.06, right=0.97, top=0.95, bottom=0.06)
                    ax1 = f.add_subplot(gs[0])
                    ax2 = f.add_subplot(gs[1])
                    if os.path.exists(goessav):
                        goes = readsav(goessav)
                        # IDL anytim 0 sec correspond to 1979 Jan 01, convert to mjd time
                        anytimbase = qa.convert(qa.quantity('1979/01/01/00:00:00'), 's')['value']
                        mjdbase = goes['utbase'] + anytimbase
                        ts = goes['tarray'] + mjdbase
                        lc0 = goes['yclean'][0, :]
                        lc1 = goes['yclean'][1, :]
                else:
                    ax1 = f.add_subplot(211)
                tstart = tim[tidx[0]]
                tend = tim[tidx[-1]]
                tstartstr = qa.time(qa.quantity(tstart, 's'))[0]
                tendstr = qa.time(qa.quantity(tend, 's'))[0]
                nfrm = int((tend - tstart) / dtframe) + 1
                print 'Movie mode set. ' + str(nfrm) + ' frames to plot from ' + tstartstr + ' to ' + tendstr
                for i in range(nfrm):
                    if (i != 0) and (i % 10 == 0):
                        print str(i) + ' frames done'
                    timeran = [tstart + i * dtframe, tstart + i * dtframe + framedur]
                    tidx1 = np.where((tim >= timeran[0]) & (tim <= timeran[1]))[0]
                    tim1 = tim[tidx1]
                    freq1 = freq[fidx] / 1e9
                    # the following is wrong
                    spec_plt1 = spec_plt[fidx, :][:, tidx1]
                    ax1.pcolormesh(tim1, freq1, spec_plt1, cmap='jet', vmin=dmin, vmax=dmax)
                    ax1.set_xlim(tim1[0], tim1[-1])
                    ax1.set_ylim(freq1[0], freq1[-1])
                    ax1.set_ylabel('Frequency (GHz)')
                    ax1.set_title('Dynamic spectrum @ bl ' + bl.split(';')[b] + ', pol ' + pol)
                    if timestr:
                        labels = ax1.get_xticks().tolist()
                        newlabels = [qa.time(qa.quantity(lb, 's'))[0] for lb in labels]
                        ax1.set_xticklabels(newlabels)
                    ax1.set_autoscale_on(False)
                    if goessav:
                        if goes_trange:
                            if type(goes_trange[0]) is str:
                                goes_trange = [qa.convert(qa.quantity(t), 's')['value'] for t in goes_trange]
                                idx = np.where((ts >= goes_trange[0]) & (ts <= goes_trange[1]))[0]
                        else:
                            idx = range(len(ts))
                        ts_plt = ts[idx]
                        lc0_plt = lc0[idx]
                        ts_plt_dt = [datetime.strptime(qa.time(qa.quantity(t, 's'), form='ymd')[0], '%Y/%m/%d/%H:%M:%S')
                                     for t in ts_plt]
                        utbase = qa.convert(qa.quantity('0001/01/01/00:00:00'), 'd')['value'] + 1
                        ts_plt_d = ts_plt / 3600. / 24. - utbase
                        # ts_plt_d = dates.date2num(ts_plt_dt)
                        ax2.plot_date(ts_plt_d, lc0_plt, 'b-')
                        ax2.axvspan(tim1[0] / 3600. / 24. - utbase, tim1[-1] / 3600. / 24. - utbase, color='red',
                                    alpha=0.5)
                        ax2.set_yscale('log')
                        # ax2.set_xlim(goes_trange)
                        # labels=ax2.get_xticks().tolist()
                        # newlabels=[qa.time(qa.quantity(lb,'s'))[0] for lb in labels]
                        # ax2.set_xticklabels(newlabels)
                        ax2.set_title('GOES 1-8 A')

                    tstartstr_ = qa.time(qa.quantity(tim1[0], 's'))[0]
                    tendstr_ = qa.time(qa.quantity(tim1[-1], 's'))[0]
                    timstr = tstartstr_.replace(':', '') + '-' + tendstr_.replace(':', '')
                    figfile = 'dspec_t' + timstr + '.png'
                    if not os.path.isdir('dspec'):
                        os.makedirs('dspec')
                    f.savefig('dspec/' + figfile)
                    plt.cla()
            else:
                f = plt.figure(figsize=(8, 4), dpi=100)
                ax = f.add_subplot(111)
                freqghz = freq / 1e9
                ax.pcolormesh(tim, freqghz, spec_plt, cmap='jet', vmin=dmin, vmax=dmax)
                ax.set_xlim(tim[tidx[0]], tim[tidx[-1]])
                ax.set_ylim(freqghz[fidx[0]], freqghz[fidx[-1]])

                def format_coord(x, y):
                    col = np.argmin(np.absolute(tim - x))
                    row = np.argmin(np.absolute(freqghz - y))
                    if col >= 0 and col < ntim and row >= 0 and row < nfreq:
                        timv = tim[col]
                        timstr = qa.time(qa.quantity(timv, 's'), form='clean', prec=9)[0]
                        flux = spec_plt[row, col]
                        return 'time {0} = {1}, freq = {2:.3f} GHz, flux = {3:.2f} Jy/beam'.format(col, timstr, y, flux)
                    else:
                        return 'x = {0}, y = {1:.3f}'.format(x, y)

                ax.format_coord = format_coord
                ax.set_ylabel('Frequency (GHz)')
                ax.set_title('Dynamic spectrum @ bl ' + bl.split(';')[b] + ', pol ' + pol)
                if timestr:
                    labels = ax.get_xticks().tolist()
                    newlabels = [qa.time(qa.quantity(lb, 's'))[0] for lb in labels]
                    ax.set_xticklabels(newlabels)
                ax.set_autoscale_on(False)

        else:
            f = plt.figure(figsize=(8, 6), dpi=100)
            R_plot = np.absolute(spec[0, b, :, :])
            L_plot = np.absolute(spec[1, b, :, :])
            I_plot = (R_plot + L_plot) / 2.
            V_plot = (R_plot - L_plot) / 2.
            if pol == 'RRLL':
                spec_plt_1 = R_plot
                spec_plt_2 = L_plot
                polstr = ['RR', 'LL']
            if pol == 'IV':
                spec_plt_1 = I_plot
                spec_plt_2 = V_plot
                polstr = ['I', 'V']

            ax1 = f.add_subplot(211)
            freqghz = freq / 1e9
            ax1.pcolormesh(tim, freqghz, spec_plt_1, cmap='jet', vmin=dmin, vmax=dmax)
            ax1.set_xlim(tim[tidx[0]], tim[tidx[-1]])
            ax1.set_ylim(freqghz[fidx[0]], freqghz[fidx[-1]])

            def format_coord(x, y):
                col = np.argmin(np.absolute(tim - x))
                row = np.argmin(np.absolute(freqghz - y))
                if col >= 0 and col < ntim and row >= 0 and row < nfreq:
                    timv = tim[col]
                    timstr = qa.time(qa.quantity(timv, 's'), form='clean', prec=9)[0]
                    flux = spec_plt[row, col]
                    return 'time {0} = {1}, freq = {2:.3f} GHz, flux = {3:.2f} Jy/beam'.format(col, timstr, y, flux)
                else:
                    return 'x = {0}, y = {1:.3f}'.format(x, y)

            ax1.format_coord = format_coord
            ax1.set_ylabel('Frequency (GHz)')
            if timestr:
                labels = ax1.get_xticks().tolist()
                newlabels = [qa.time(qa.quantity(lb, 's'))[0] for lb in labels]
                ax1.set_xticklabels(newlabels)
            ax1.set_title('Dynamic spectrum @ bl ' + bl.split(';')[b] + ', pol ' + polstr[0])
            ax1.set_autoscale_on(False)
            ax2 = f.add_subplot(212)
            ax2.pcolormesh(tim, freqghz, spec_plt_2, cmap='jet', vmin=dmin, vmax=dmax)
            ax2.set_xlim(tim[tidx[0]], tim[tidx[-1]])
            ax2.set_ylim(freqghz[fidx[0]], freqghz[fidx[-1]])
            if timestr:
                labels = ax2.get_xticks().tolist()
                newlabels = [qa.time(qa.quantity(lb, 's'))[0] for lb in labels]
                ax2.set_xticklabels(newlabels)

            def format_coord(x, y):
                col = np.argmin(np.absolute(tim - x))
                row = np.argmin(np.absolute(freqghz - y))
                if col >= 0 and col < ntim and row >= 0 and row < nfreq:
                    timv = tim[col]
                    timstr = qa.time(qa.quantity(timv, 's'), form='clean', prec=9)[0]
                    flux = spec_plt[row, col]
                    return 'time {0} = {1}, freq = {2:.3f} GHz, flux = {3:.2f} Jy/beam'.format(col, timstr, y, flux)
                else:
                    return 'x = {0}, y = {1:.3f}'.format(x, y)

            ax2.format_coord = format_coord
            ax2.set_ylabel('Frequency (GHz)')
            ax2.set_title('Dynamic spectrum @ bl ' + bl.split(';')[b] + ', pol ' + polstr[1])
            ax2.set_autoscale_on(False)
        if savepng:
            figfile = specfile[:(specfile.find('npz'))] + 'png'
            f.savefig(figfile)
        if savepdf:
            figfile = specfile[:(specfile.find('npz'))] + 'pdf'
            f.savefig(figfile)
            # plt.close()
            # return (f,ax)


def wrt_dspec(specfile=None, specdat=None):
    try:
        specfile
    except NameError:
        print 'No input centfile specified for reading. Aborting...'
    if not specdat:
        print 'Output file name is not specified, use the default convention'
        specdat = specfile.replace('npz', 'dat')
    specdata = np.load(specfile)
    spec = np.copy(specdata['spec'][:, :, :, :])
    npl, nbl, nf, nt = spec.shape
    print 'Dimension of the data cube -- # of pol, # of baseline, # of frequency, # of time:'
    print npl, nbl, nf, nt
    nele = npl * nbl * nf * nt
    # need to transpose to nf, nt, npl, nbl
    spec = np.transpose(spec, (2, 3, 0, 1))
    # Turn fmx into a 1d array for writing
    spec = spec.flatten()
    # Pack the data into a byte buffer.  Note the asterisks in the next three lines!
    # If your data includes integers or other data types, use 'i' or other instead of 'f'
    buf = struct.pack(str(nf) + 'f', *specdata['freq'])
    buf += struct.pack(str(nt) + 'd', *specdata['tim'])
    buf += struct.pack(str(nele) + 'f', *spec)
    with open(specdat, 'wb') as f:
        f.write(buf)
    f.close()


def plt_dspec_old(mspath=None, specfile=None, pol='I', dmin=None, dmax=None, fig=None):
    # Set up variables 
    if pol != 'RR' and pol != 'LL' and pol != 'RRLL' and pol != 'I' and pol != 'V' and pol != 'IV':
        print "Please enter 'RR', 'LL', 'RRLL', 'I', 'V', 'IV' for pol"
        return 0

    if not mspath:
        mspath = '.'
    specdata = np.load(mspath + '/' + specfile)
    spec = specdata['spec']
    tim = specdata['tim']
    freq = specdata['freq']

    # setup plot parameters
    print 'ploting dynamic spectrum...'
    # mask the channels from 512 to 519 (no observation)
    spec = np.ma.array(spec)
    spec[:, 512:519, :] = np.ma.masked
    spec_med = np.median(np.absolute(spec))
    # set the time axis
    ntim = tim.shape
    if ntim[0] < 20:
        xticks = np.arange((ntim[0] - 1) / 2 + 1) * 2
    elif ntim[0] < 100:
        xticks = np.arange((ntim[0] - 1) / 20 + 1) * 20
    elif ntim[0] < 600:
        xticks = np.arange((ntim[0] - 1) / 100 + 1) * 100
    elif ntim[0] < 2400:
        xticks = np.arange((ntim[0] - 1) / 400 + 1) * 400
    elif ntim[0] < 12000:
        # 1 min per step
        tstart = np.fix(tim[0] / 60.) + 1
        xstart = np.abs(tim - tstart * 60.).argmin()
        xticks = np.arange(ntim[0] / 1200) * 1200 + xstart
    elif ntim[0] > 12000:
        xticks = np.arange(ntim[0] / 6000 + 1) * 6000
    nticks = xticks.shape
    xticktims = []
    for i in range(nticks[0]):
        # xticktim0=qa.time(qa.quantity(tim[xticks[i]],'s'))
        tims = tim[xticks[i]]  # in seconds
        tims_jd = jdutil.mjd_to_jd(tims / 3600. / 24.)  # to julian date
        tims_dt = jdutil.jd_to_datetime(tims_jd)
        tims_dt2 = tims_dt + datetime.timedelta(seconds=round(tims_dt.microsecond / 1e6))
        tims_char = tims_dt2.strftime('%H:%M:%S')
        xticktims.append(tims_char)
    xticks = list(xticks)
    # do the plot
    f = plt.figure(figsize=(10, 6), dpi=100)
    if not dmin:
        dmin = spec_med / 20.
    if not dmax:
        dmax = spec_med * 5.
    if pol != 'RRLL' and pol != 'IV':
        ax = f.add_subplot(111)
        if pol == 'RR':
            spec_plt = np.absolute(spec[0, :, :])
        elif pol == 'LL':
            spec_plt = np.absolute(spec[1, :, :])
        elif pol == 'I':
            spec_plt = (np.absolute(spec[0, :, :]) + np.absolute(spec[1, :, :])) / 2.
        elif pol == 'V':
            spec_plt = (np.absolute(spec[0, :, :]) - np.absolute(spec[1, :, :])) / 2.
        # ax.imshow(spec_plt,aspect='auto',origin='lower',extent=[0,ntim[0]-1,np.min(freq)/1e6,np.max(freq)/1e6],\
        #          vmin=dmin,vmax=dmax,interpolation='none')
        # ax.set_xticks(xticks)
        # ax.set_xticklabels(xticktims)
        tim_ = tim - tim[0]
        freqghz = freq / 1e9
        ax.pcolormesh(tim_, freqghz, spec_plt, cmap='jet', vmin=dmin, vmax=dmax)
        ax.set_xlim([tim_[0], tim_[-1]])
        ax.set_ylim([freqghz[0], freqghz[-1]])
        ax.set_xlabel('Time (pixel)')
        ax.set_ylabel('Frequency (GHz)')
        ax.set_title('VLA dynamic spectrum for pol ' + pol)
        ax.set_autoscale_on(False)
    else:
        R_plot = np.absolute(spec[0, :, :])
        L_plot = np.absolute(spec[1, :, :])
        I_plot = (Rplot + Lplot) / 2.
        V_plot = (Rplot - Lplot) / 2.
        if pol == 'RRLL':
            spec_plt_1 = R_plot
            spec_plt_2 = L_plot
        if pol == 'IV':
            spec_plt_1 = I_plot
            spec_plt_2 = V_plot

        ax1 = f.add_subplot(211)
        ax1.imshow(spec_plt_1, aspect='auto', origin='lower',
                   extent=[0, ntim[0] - 1, np.min(freq) / 1e6, np.max(freq) / 1e6], \
                   vmin=dmin, vmax=dmax, interpolation='none')
        ax1.set_xticks(xticks)
        ax1.set_xticklabels(xticktims)
        ax1.set_xlabel('Universal Time (50 ms/pixel)')
        ax1.set_ylabel('Frequency (MHz)')
        ax2.set_title('VLA dynm spec for pol ' + pol[0])
        ax1.set_autoscale_on(False)
        ax2 = f.add_subplot(212)
        ax2.imshow(spec_plt_2, aspect='auto', origin='lower',
                   extent=[0, ntim[0] - 1, np.min(freq) / 1e6, np.max(freq) / 1e6], \
                   vmin=dmin, vmax=dmax, interpolation='none')
        ax2.set_xticks(xticks)
        ax2.set_xticklabels(xticktims)
        ax2.set_xlabel('Universal Time (50 ms/pixel)')
        ax2.set_ylabel('Frequency (MHz)')
        ax2.set_title('VLA dynm spec for pol ' + pol[-1])
        ax2.set_autoscale_on(False)
    if fig:
        figfile = specfile[:(specfile.find('spec'))] + pol + '.pdf'
        f.savefig(mspath + '/' + figfile)
    # plt.close()
    return (f, ax)



#!/usr/bin/env python

"""
Function for interactively selecting part of an array displayed as an image with matplotlib.
"""

import matplotlib.pyplot as plt
from matplotlib import is_interactive
from matplotlib.path import Path
from matplotlib.widgets import LassoSelector, RectangleSelector
import numpy as np


def path_bbox(p):
    """
    Return rectangular bounding box of given path.
    Parameters
    ----------
    p : array_like
        Array of vertices with shape Nx2.
    Returns
    -------
    bbox : array_like
        Array of bounding box vertices with shape 4x2.
    """

    assert p.ndim == 2
    assert p.shape[1] == 2

    ix_min = p[:, 0].argmin()
    ix_max = p[:, 0].argmax()
    iy_min = p[:, 1].argmin()
    iy_max = p[:, 1].argmax()

    return np.array([[p[ix_min, 0], p[iy_min, 1]], [p[ix_min, 0], p[iy_max, 1]], [p[ix_max, 0], p[iy_max, 1]], [p[ix_max, 0], p[iy_min, 1]]])


def imshow_select(data, selected=None, selector='lasso', bbox=False, sav_img=False, contour_color='w', plotonly=False):
    """
    Display array as image with region selector.

    Parameters
    ----------
    data : array_like
        Array to display.
    selector : str
        Region selector. For `lasso`, use `LassoSelector`; for `rectangle`,
        use `RectangleSelector`.
    bbox : bool
        If True, only return array within rectangular bounding box of selected region.
        Otherwise, return array with same dimensions as `data` such that selected region
        contains the corresponding values from `data` and the remainder contains 0.
    Returns
    -------
    region : array_like
        Data for selected region.
    mask : array_like
        Boolean mask with same shape of `data` for selecting the returned region from `data`.
    """

    interactive = is_interactive()
    if not interactive:
        plt.ion()
    fig, axs = plt.subplots(ncols=2, nrows=1, sharex=True, sharey=True, subplot_kw={'adjustable': 'box-forced'})
    axs = axs.ravel()
    axs[0].imshow(data, origin='lower')
    axs[1].imshow(np.zeros_like(data), origin='lower')
    plt.subplots_adjust()

    x, y = np.meshgrid(np.arange(data.shape[1], dtype=int), np.arange(data.shape[0], dtype=int))
    pix = np.vstack((x.flatten(), y.flatten())).T

    # Store data in dict value to permit overwriting by nested
    # functions in Python 2.7:
    collects = {}
    if selected is None:
        selected = {}
    else:
        if 'mask' in selected.keys():
            collects['contour_ax_orig'] = axs[0].contour(selected['mask'].astype(np.int), levels=[0.5], linewidths=0.75, colors=contour_color)
            if 'data' not in selected.keys():
                selected['data'] = np.zeros_like(data)
                selected['data'][selected['mask']] = data[selected['mask']]
            axs[1].imshow(selected['data'], origin='lower')
            fig.canvas.draw_idle()
        else:
            selected = {}

    def _onselect_lasso(verts):
        verts = np.array(verts)
        p = Path(verts)
        ind = p.contains_points(pix, radius=0.5)
        if 'data' in selected.keys() and (not bbox):
            selected['data'].flat[ind] = data.flat[ind]
            selected['mask'].flat[ind] = True
            if 'contour_ax_orig' in collects.keys():
                for coll in collects['contour_ax_orig'].collections:
                    axs[0].collections.remove(coll)
                collects['contour_ax_orig'] = axs[0].contour(selected['mask'].astype(np.int), levels=[0.5], linewidths=0.75, colors=contour_color)
        else:
            selected['data'] = np.zeros_like(data)
            selected['mask'] = np.tile(False, data.shape)
            selected['data'].flat[ind] = data.flat[ind]
            selected['mask'].flat[ind] = True
            collects['contour_ax_orig'] = axs[0].contour(selected['mask'].astype(np.int), levels=[0.5], linewidths=0.75, colors=contour_color)
        axs[1].imshow(selected['data'], origin='lower')
        fig.canvas.draw_idle()
        if bbox:
            b = path_bbox(verts)
            selected['data'] = selected['data'][int(min(b[:, 1])):int(max(b[:, 1])), int(min(b[:, 0])):int(max(b[:, 0]))]

    def _deselect_lasso(verts):
        verts = np.array(verts)
        p = Path(verts)
        ind = p.contains_points(pix, radius=0.5)
        if 'data' in selected.keys():
            selected['data'].flat[ind] = 0.
            selected['mask'].flat[ind] = False
            if 'contour_ax_orig' in collects.keys():
                for coll in collects['contour_ax_orig'].collections:
                    axs[0].collections.remove(coll)
                collects['contour_ax_orig'] = axs[0].contour(selected['mask'].astype(np.int), levels=[0.5], linewidths=0.75, colors=contour_color)
            axs[1].imshow(selected['data'], origin='lower')
            fig.canvas.draw_idle()

    def _onselect_rectangle(start, end):
        verts = np.array([[start.xdata, start.ydata], [start.xdata, end.ydata], [end.xdata, end.ydata], [end.xdata, start.ydata]], int)
        p = Path(verts)
        ind = p.contains_points(pix, radius=0.5)
        if 'data' in selected.keys() and (not bbox):
            selected['data'].flat[ind] = data.flat[ind]
            selected['mask'].flat[ind] = True
            if 'contour_ax_orig' in collects.keys():
                for coll in collects['contour_ax_orig'].collections:
                    axs[0].collections.remove(coll)
                collects['contour_ax_orig'] = axs[0].contour(selected['mask'].astype(np.int), levels=[0.5], linewidths=0.75, colors=contour_color)
        else:
            selected['data'] = np.zeros_like(data)
            selected['mask'] = np.tile(False, data.shape)
            selected['data'].flat[ind] = data.flat[ind]
            selected['mask'].flat[ind] = True
            collects['contour_ax_orig'] = axs[0].contour(selected['mask'].astype(np.int), levels=[0.5], linewidths=0.75, colors=contour_color)
        axs[1].imshow(selected['data'], origin='lower')
        fig.canvas.draw_idle()
        if bbox:
            b = path_bbox(verts)
            selected['data'] = selected['data'][min(b[:, 1]):max(b[:, 1]), min(b[:, 0]):max(b[:, 0])]

    def _deselect_rectangle(start, end):
        verts = np.array([[start.xdata, start.ydata], [start.xdata, end.ydata], [end.xdata, end.ydata], [end.xdata, start.ydata]], int)
        p = Path(verts)
        ind = p.contains_points(pix, radius=0.5)
        if 'data' in selected.keys():
            selected['data'].flat[ind] = 0.
            selected['mask'].flat[ind] = False
            if 'contour_ax_orig' in collects.keys():
                for coll in collects['contour_ax_orig'].collections:
                    axs[0].collections.remove(coll)
                collects['contour_ax_orig'] = axs[0].contour(selected['mask'].astype(np.int), levels=[0.5], linewidths=0.75, colors=contour_color)
            axs[1].imshow(selected['data'], origin='lower')
            fig.canvas.draw_idle()

    name_to_selector = {'lasso': LassoSelector, 'rectangle': RectangleSelector}
    selector = name_to_selector[selector]
    onselect_dict = {LassoSelector: _onselect_lasso, RectangleSelector: _onselect_rectangle}
    deselect_dict = {LassoSelector: _deselect_lasso, RectangleSelector: _deselect_rectangle}
    kwargs_dict = {LassoSelector: {}, RectangleSelector: {'interactive': True}}

    lasso = selector(axs[0], onselect_dict[selector], **kwargs_dict[selector])
    lasso2 = selector(axs[1], deselect_dict[selector], **kwargs_dict[selector])
    if not plotonly:
        raw_input('Press Enter when done')
    lasso.disconnect_events()
    lasso2.disconnect_events()
    if not interactive:
        plt.ioff()
    if sav_img:
        if type(sav_img) is str:
            fig.savefig(sav_img)
        else:
            fig.savefig('selected.png')
    return selected['data'], selected['mask']

# if __name__ == '__main__':
#     from skimage.data import coins
#
#     data = coins()
#     selected, mask = imshow_select(data, 'lasso', True)
#     plt.imsave('selected.png', selected)
#     plt.imsave('mask.png', mask)



import numpy as np
# import sys
import math
import os, sys, platform
import astropy.units as u
from sunpy import map as smap
from astropy.coordinates import SkyCoord
from suncasa.io import ndfits
from . import gstools  # initialization library - located either in the current directory or in the system path
from suncasa.utils import mstools
import lmfit
from astropy.time import Time
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import matplotlib.colorbar as colorbar
from suncasa.utils import mstools
from suncasa.utils import qlookplot as ql
from mpl_toolkits.axes_grid1 import make_axes_locatable
from tqdm import tqdm
from astropy.io import fits
import numpy.ma as ma


# name of the fast gyrosynchrotron codes shared library
if platform.system() == 'Linux' or platform.system() == 'Darwin':
    libname = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                           'binaries/MWTransferArr.so')
if platform.system() == 'Windows':
    libname = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                           'binaries/MWTransferArr64.dll')


def kev2k(eng):
    return 11604525.00617 * eng


def ff_emission(em, T=1.e7, Z=1., mu=1.e10):
    from astropy import constants as const
    import astropy.units as u

    T = T * u.k
    mu = mu * u.Hz
    esu = const.e.esu
    k_B = const.k_B.cgs
    m_e = const.m_e.cgs
    c = const.c.cgs
    bmax = (3 * k_B * T * u.k / m_e) ** 0.5 / 2.0 / np.pi / (mu * u.Hz)
    bmin = Z * esu ** 2 / 3. / k_B / T
    lnbb = np.log((bmax / bmin).value)
    ka_mu = 1. / mu ** 2 / T ** 1.5 * (
            Z ** 2 * esu ** 6 / c / np.sqrt(2. * np.pi * (m_e * k_B) ** 3)) * np.pi ** 2 / 4.0 * lnbb
    # print(ka_mu, em)
    opc = ka_mu * em
    return T.value * (1 - np.exp(-opc.value))


def sfu2tb(freq, flux, area):
    # frequency in Hz
    # flux in sfu
    # area: area of the radio source in arcsec^2
    sfu2cgs = 1e-19
    vc = 2.998e10
    kb = 1.38065e-16
    # sr = np.pi * (size[0] / 206265. / 2.) * (size[1] / 206265. / 2.)
    sr = area / 206265. ** 2
    Tb = flux * sfu2cgs * vc ** 2. / (2. * kb * freq ** 2. * sr)
    return Tb


def tb2sfu(freq, tb, area):
    # frequency in Hz
    # brightness temperature in K
    # area: area of the radio source in arcsec^2
    sfu2cgs = 1e-19
    vc = 2.998e10
    kb = 1.38065e-16
    # sr = np.pi * (size[0] / 206265. / 2.) * (size[1] / 206265. / 2.)
    sr = area / 206265. ** 2
    flux = tb / (sfu2cgs * vc ** 2. / (2. * kb * freq ** 2. * sr))
    return flux


def initspecplot(axes, cplts):
    errobjs = []
    for cpltidx, cplt in enumerate(cplts):
        errobjs.append(axes.errorbar([], [], yerr=[], linestyle='', marker='o', mfc='none', mec=cplt, alpha=1.0))

    axes.set_yscale("log")
    axes.set_xscale("log")
    axes.set_xlim([1, 20])
    axes.set_ylim([0.1, 1000])
    axes.set_xticks([1, 5, 10, 20])
    axes.set_xticklabels([1, 5, 10, 20])
    axes.set_xticks([1, 5, 10, 20])
    axes.set_yticks([])
    axes.set_yticks([0.01, 0.1, 1, 10, 100, 1000])
    axes.set_ylabel('T$_b$ [MK]')
    axes.set_xlabel('Frequency [GHz]')

    x = np.linspace(1, 20, 10)
    for ll in [-1, 0, 1, 2, 3, 4]:
        y = 10. ** (-2 * np.log10(x) + ll)
        axes.plot(x, y, 'k--', alpha=0.1)
        # y2 = 10. ** (-4 * np.log10(x) + ll)
        # y3 = 10. ** (-8 * np.log10(x) + ll)
        # ax_eospec.plot(x, y, 'k--', x, y2, 'k:', x, y3, 'k-.', alpha=0.1)
    return errobjs


def set_errorobj(xout, yout, errobj, yerr=None):
    eospec, dummy, (errbar_eospec,) = errobj
    eospec.set_data(xout, yout)
    if yerr is not None:
        yerr_top = yout + yerr
        yerr_bot = yout - yerr
        new_segments_y = [np.array([[x, yt], [x, yb]]) for x, yt, yb in zip(xout, yerr_top, yerr_bot)]
        errbar_eospec.set_segments(new_segments_y)


def mwspec2min_1src(params, freqghz, tb=None, tb_err=None, arcsec2cm=0.725e8, showplt=False):
    # params are defined by lmfit.Paramters()
    '''
    params: parameters defined by lmfit.Paramters()
    freqghz: frequencies in GHz
    ssz: pixel size in arcsec
    tb: reference brightness temperature in K
    tb_err: uncertainties of reference brightness temperature in K
    '''

    from scipy import interpolate
    GET_MW = gstools.initGET_MW(libname)  # load the library

    ssz = float(params['ssz'].value)  # # source area in arcsec^2
    depth = float(params['depth'].value)  # total source depth in arcsec
    Bmag = float(params['Bmag'].value)  # magnetic field strength in G
    Tth = float(params['Tth'].value)  # thermal temperature in MK
    nth = float(params['nth'].value)  # thermal density in 1e10 cm^{-3}
    nrlh = 10. ** float(params['lognrlh'].value)  # total nonthermal density above 0.1 MeV
    delta = float(params['delta'].value)  # powerlaw index
    theta = float(params['theta'].value)  # viewing angle in degrees
    Emin = float(params['Emin'].value)  # low energy cutoff of nonthermal electrons in MeV
    Emax = float(params['Emax'].value)  # high energy cutoff of nonthermal electrons in MeV
    E_hi = 0.1
    nrl = nrlh * (Emin ** (1. - delta) - Emax * (1. - delta)) / (E_hi ** (1. - delta) - Emax ** (1. - delta))

    Nf = 100  # number of frequencies
    NSteps = 1  # number of nodes along the line-of-sight

    N_E = 15  # number of energy nodes
    N_mu = 15  # number of pitch-angle nodes

    Lparms = np.zeros(11, dtype='int32')  # array of dimensions etc.
    Lparms[0] = NSteps
    Lparms[1] = Nf
    Lparms[2] = N_E
    Lparms[3] = N_mu

    Rparms = np.zeros(5, dtype='double')  # array of global floating-point parameters
    Rparms[0] = ssz * arcsec2cm ** 2  # Area, cm^2
    # Rparms[0] = 1e20  # area, cm^2
    Rparms[1] = 1e9  # starting frequency to calculate spectrum, Hz
    Rparms[2] = 0.02  # logarithmic step in frequency
    Rparms[3] = 12  # f^C
    Rparms[4] = 12  # f^WH

    ParmLocal = np.zeros(24, dtype='double')  # array of voxel parameters - for a single voxel
    ParmLocal[0] = depth * arcsec2cm / NSteps  # voxel depth, cm
    ParmLocal[1] = Tth * 1e6  # T_0, K
    ParmLocal[2] = nth * 1e10  # n_0 - thermal electron density, cm^{-3}
    ParmLocal[3] = Bmag  # B - magnetic field, G

    Parms = np.zeros((24, NSteps), dtype='double', order='F')  # 2D array of input parameters - for multiple voxels
    for i in range(NSteps):
        Parms[:, i] = ParmLocal  # most of the parameters are the same in all voxels
        # if NSteps > 1:
        #     Parms[4, i] = 50.0 + 30.0 * i / (NSteps - 1)  # the viewing angle varies from 50 to 80 degrees along the LOS
        # else:
        #     Parms[4, i] = 50.0  # the viewing angle varies from 50 to 80 degrees along the LOS
        Parms[4, i] = theta

    # parameters of the electron distribution function
    n_b = nrl  # n_b - nonthermal electron density, cm^{-3}
    mu_c = np.cos(np.pi * 70 / 180)  # loss-cone boundary
    dmu_c = 0.2  # Delta_mu

    E_arr = np.logspace(np.log10(Emin), np.log10(Emax), N_E, dtype='double')  # energy grid (logarithmically spaced)
    mu_arr = np.linspace(-1.0, 1.0, N_mu, dtype='double')  # pitch-angle grid

    f0 = np.zeros((N_E, N_mu), dtype='double')  # 2D distribution function array - for a single voxel

    # computing the distribution function (equivalent to PLW & GLC)
    A = n_b / (2.0 * np.pi) * (delta - 1.0) / (Emin ** (1.0 - delta) - Emax ** (1.0 - delta))
    B = 0.5 / (mu_c + dmu_c * np.sqrt(np.pi) / 2 * math.erf((1.0 - mu_c) / dmu_c))
    for i in range(N_E):
        for j in range(N_mu):
            amu = abs(mu_arr[j])
            f0[i, j] = A * B * E_arr[i] ** (-delta) * (1.0 if amu < mu_c else np.exp(-((amu - mu_c) / dmu_c) ** 2))

    f_arr = np.zeros((N_E, N_mu, NSteps), dtype='double',
                     order='F')  # 3D distribution function array - for multiple voxels
    for k in range(NSteps):
        f_arr[:, :, k] = f0  # electron distribution function is the same in all voxels

    RL = np.zeros((7, Nf), dtype='double', order='F')  # input/output array

    # calculating the emission for array distribution (array -> on)
    res = GET_MW(Lparms, Rparms, Parms, E_arr, mu_arr, f_arr, RL)

    if res:
        # retrieving the results
        f = RL[0]
        I_L = RL[5]
        I_R = RL[6]

        if showplt:
            import matplotlib.pyplot as plt
            fig, ax = plt.subplots(1, 1)
            ax.plot(f, I_L + I_R)
            ax.set_xscale('log')
            ax.set_yscale('log')
            ax.set_title('Total intensity (array)')
            ax.set_xlabel('Frequency, GHz')
            ax.set_ylabel('Intensity, sfu')

        flx_model = I_L + I_R
        flx_model = np.nan_to_num(flx_model) + 1e-11
        logf = np.log10(f)
        logflx_model = np.log10(flx_model)
        logfreqghz = np.log10(freqghz)
        interpfunc = interpolate.interp1d(logf, logflx_model, kind='linear')
        logmflx = interpfunc(logfreqghz)
        mflx = 10. ** logmflx
        mtb = sfu2tb(np.array(freqghz) * 1.e9, mflx, ssz)
    else:
        print("Calculation error!")

    if tb is None:
        return mtb
    if tb_err is None:
        # return mTb - Tb
        return mtb - tb
    # wt = 1./flx_err
    # wt = 1./(Tb_err/Tb/np.log(10.))
    # residual = np.abs((logmTb - np.log10(Tb))) * wt
    # residual = np.abs((mflx - flx)) * wt
    residual = (mtb - tb) / tb_err
    return residual


class RegionSelector:

    # def set_errorobj(self, xout, yout, errobj, yerr):
    #     eospec, dummy, (errbar_eospec,) = errobj
    #     eospec.set_data(xout, yout)
    #     if yerr is not None:
    #         yerr_top = yout + yerr
    #         yerr_bot = yout - yerr
    #         new_segments_y = [np.array([[x, yt], [x, yb]]) for x, yt, yb in zip(xout, yerr_top, yerr_bot)]
    #         errbar_eospec.set_segments(new_segments_y)
    #     return 1

    def subdata(self, xs, ys, rfile):
        rmap, rdata, rheader, ndim, npol_fits, stokaxis, rfreqs, rdelts = ndfits.read(rfile)
        ny, nx = rmap.data.shape
        tr_coord = rmap.top_right_coord
        bl_coord = rmap.bottom_left_coord
        x0 = bl_coord.Tx.to(u.arcsec).value
        y0 = bl_coord.Ty.to(u.arcsec).value
        x1 = tr_coord.Tx.to(u.arcsec).value
        y1 = tr_coord.Ty.to(u.arcsec).value
        dx = rmap.scale.axis1.to(u.arcsec / u.pix).value
        dy = rmap.scale.axis2.to(u.arcsec / u.pix).value
        mapx, mapy = np.linspace(x0, x1, nx) - dx / 2.0, np.linspace(y0, y1, ny) - dy / 2.0
        xsmin = np.nanmin(xs)
        xsmax = np.nanmax(xs)
        ysmin = np.nanmin(ys)
        ysmax = np.nanmax(ys)
        if np.abs(xsmax - xsmin) < dx:
            xsmax = xsmin + dx
        if np.abs(ysmax - ysmin) < dy:
            ysmax = ysmin + dy
        xmask = np.logical_and(mapx >= xsmin, mapx <= xsmax)
        nxnew = np.count_nonzero(xmask)
        ymask = np.logical_and(mapy >= ysmin, mapy <= ysmax)
        nynew = np.count_nonzero(ymask)
        xmask = np.tile(xmask, ny).reshape(ny, nx)
        ymask = np.tile(ymask, nx).reshape(nx, ny).transpose()
        mask = xmask & ymask
        # print(np.count_nonzero(mask))
        self.npix = np.count_nonzero(mask)
        self.area = self.npix * dx * dy
        data = rdata[:, mask]
        # print(rdata[:, :, mask])
        # print(mask.shape, rdata.shape, data.shape)
        data = np.squeeze(data)
        # print(data.shape)
        return data

    def __init__(self, clkpnts, boxlines, eofiles, errobjs, cfreqs=None, rms=None, eofile_ref=None, errobj_ref=None,
                 wTmap=None, outspec_ff=None, scatter_gsfit=None,
                 get_peak=False, get_sum=False):
        self.boxline = []
        self.clkpnt = []
        self.xs = list(clkpnts[0].get_xdata())
        self.ys = list(clkpnts[0].get_ydata())
        self.npix = None
        self.area = None
        self.xout = []
        self.yout = []
        self.xouterr = []
        self.youterr = []
        for errobj in errobjs:
            eospec, dummy, (errbar_eospec,) = errobj
            self.xout.append(eospec.get_xdata())
            self.yout.append(eospec.get_ydata())
        self.errobjs = errobjs
        self.errobj_ref = errobj_ref
        self.outspec_ff = outspec_ff
        self.scatter_gsfit = scatter_gsfit
        self.cfreqs = cfreqs
        self.rms = rms
        self.eofiles = eofiles
        self.eofile_ref = eofile_ref
        self.wTmap = wTmap
        self.wT = None
        self.em = None
        self.get_peak = get_peak
        self.get_sum = get_sum
        self.tps = []
        self.params = None
        for idx, s in enumerate(clkpnts):
            self.boxline.append(boxlines[idx])
            self.clkpnt.append(s)
            self.cid = s.figure.canvas.mpl_connect('button_press_event', self)

    def __call__(self, event):
        axes = [clkpnt.axes for clkpnt in self.clkpnt]
        if self.clkpnt[0].figure.canvas.toolbar.mode == '':
            if event.inaxes not in axes:
                return
            nxs = len(self.xs)
            if event.button == 1:
                if nxs < 2:
                    self.xs.append(event.xdata)
                    self.ys.append(event.ydata)
                else:
                    self.xs = [event.xdata]
                    self.ys = [event.ydata]
            elif event.button == 3:
                if len(self.xs) > 0:
                    self.xs.pop()
                    self.ys.pop()

            self.get_flux()

    def get_flux(self):
        if len(self.xs) > 0:
            xs = np.array(self.xs, dtype=np.float64)
            ys = np.array(self.ys, dtype=np.float64)
            for clkpnt in self.clkpnt:
                clkpnt.set_data(xs, ys)
        else:
            for clkpnt in self.clkpnt:
                clkpnt.set_data([], [])
        nxs = len(self.xs)
        if nxs <= 1:
            for line in self.boxline:
                line.set_data([], [])
        elif nxs == 2:
            datas = []
            # eofile = self.eofiles[0]
            # rmap, rdata, rheader, ndim, npol_fits, stokaxis, rfreqs, rdelts = ndfits.read(eofile)
            # data = self.subdata(xs, ys, eofile)
            # datas.append(data)
            for tidx, eofile in enumerate(self.eofiles):
                data = self.subdata(xs, ys, eofile)
                datas.append(data)

            if self.eofile_ref is not None:
                data_ref = self.subdata(xs, ys, self.eofile_ref)
            if self.wTmap is not None:
                datawT = self.subdata(xs, ys, self.wTmap)

            if self.get_peak:
                youts_outspec = []
                for data in datas:
                    if data.ndim > 1:
                        youts_outspec.append(np.nanmax(data, axis=-1) / 1e6)
                    else:
                        youts_outspec.append(data / 1e6)
                if self.eofile_ref is not None:
                    youts_outspec_ref = np.nanmax(data_ref[0, dd, :, :]) / 1e6
            else:
                youts_outspec = []
                for data in datas:
                    if data.ndim > 1:
                        youts_outspec.append(np.nanmean(data, axis=-1) / 1e6)
                    else:
                        youts_outspec.append(data / 1e6)
                if self.eofile_ref is not None:
                    if data.ndim > 1:
                        youts_outspec_ref = np.nanmean(data_ref, axis=-1) / 1e6
                    else:
                        youts_outspec_ref = data_ref / 1e6
            self.tps = []
            for data in datas:
                if data.ndim > 1:
                    self.tps.append(np.nansum(data, axis=-1) / 1e6)
                else:
                    self.tps.append(data / 1e6)
            xout = self.cfreqs
            for tidx, errobj in enumerate(self.errobjs):
                set_errorobj(xout, youts_outspec[tidx], errobj, self.rms)
            if self.eofile_ref is not None:
                set_errorobj(xout, youts_outspec_ref, self.errobj_ref, self.rms)
            if self.wTmap is not None:
                print(datawT.shape)
                wT = np.nanmean(datawT[..., 1]) * 1e6
                em = np.nanmean(datawT[..., 0])
                arcsec2cm = (self.wTmap[0].rsun_meters / self.wTmap[0].rsun_obs).to(u.cm / u.arcsec).value
                # nele = 4.0e10
                # depth = em / nele ** 2 / arcsec2cm
                # print('Temperature: {:.1f} MK, EM: {:.2e} cm-5, depth: {:.1f} arcsec if nele is {:.2e} cm-3'.format(wT / 1e6, em, depth, nele))
                depth = 20.  ## arcsec
                nele = np.sqrt(em / (depth * arcsec2cm))
                print('Temperature: {:.1f} MK, EM: {:.2e} cm-5, nele: {:.2e} cm-3 if depth is {:.1f} arcsec'.format(
                    wT / 1e6, em, nele, depth))
                self.wT = wT
                self.em = em
                yout_ff = np.array([ff_emission(em, T=wT, Z=1., mu=ll) for ll in xout * 1e9]) / 1.e6

                self.outspec_ff.set_data(xout, yout_ff)

            self.errobjs[0][0].figure.canvas.draw_idle()
            for line in self.boxline:
                line.set_data([xs[0], xs[1], xs[1], xs[0], xs[0]], [ys[0], ys[0], ys[1], ys[1], ys[0]])

        clkpnt.figure.canvas.draw_idle()


class GStool:
    # def get_showaia(self):
    #     return self._showaia
    #
    # def set_showaia(self, value):
    #     self._showaia = value
    #
    # showaia = property(fget=get_showaia, fset=set_showaia, doc="`Boolean`-like: Display AIA image or not")

    def __init__(self, eofiles, aiafile=None, xycen=None, fov=None, freqghz_bound=[-1, 100], calpha=0.5,
                 clevels=np.array([0.3, 1.0]), opencontour=None):
        self.aiafile = aiafile
        self.eofiles = eofiles
        self.xycen = xycen
        self.fov = fov
        self.calpha = calpha
        self.clevels = clevels
        self.freqghz_bound = freqghz_bound
        self.opencontour = opencontour
        self._showaia = False

        rmap, rdata, rheader, ndim, npol_fits, stokaxis, rfreqs, rdelts = ndfits.read(eofiles[0])
        self.bdinfo = bdinfo = ndfits.get_bdinfo(rfreqs, rdelts)
        self.cfreqs = cfreqs = bdinfo['cfreqs']
        self.cfreqs_all = cfreqs_all = bdinfo['cfreqs_all']
        self.freq_dist = lambda fq: (fq - cfreqs_all[0]) / (cfreqs_all[-1] - cfreqs_all[0])

        self.ntim = ntim = len(eofiles)
        self.xlim = xlim = xycen[0] + np.array([-1, 1]) * 0.5 * fov[0]
        self.ylim = ylim = xycen[1] + np.array([-1, 1]) * 0.5 * fov[1]

        nspw = len(rfreqs)
        eodate = Time(rmap.date.mjd + rmap.exposure_time.value / 2. / 24 / 3600, format='mjd')
        ny, nx = rmap.data.shape
        x0, x1 = (np.array([1, rmap.meta['NAXIS1']]) - rmap.meta['CRPIX1']) * rmap.meta['CDELT1'] + \
                 rmap.meta['CRVAL1']
        y0, y1 = (np.array([1, rmap.meta['NAXIS2']]) - rmap.meta['CRPIX2']) * rmap.meta['CDELT2'] + \
                 rmap.meta['CRVAL2']
        dx = rmap.meta['CDELT1']
        dy = rmap.meta['CDELT2']
        mapx, mapy = np.linspace(x0, x1, nx), np.linspace(y0, y1, ny)

        fig = plt.figure(figsize=(15, 6))
        self.fig = fig

        grids = fig.add_gridspec(ncols=3, nrows=1, width_ratios=[1, 1, 0.6])
        self.grids = grids
        axs = []
        axs.append(fig.add_subplot(grids[0, 0]))
        axs.append(fig.add_subplot(grids[0, 1], sharex=axs[-1], sharey=axs[-1]))
        axs.append(fig.add_subplot(grids[0, 2]))
        if aiafile:
            if os.path.exists(aiafile):
                try:
                    aiacmap = plt.get_cmap('gray_r')
                    aiamap = smap.Map(aiafile)
                    ax = axs[0]
                    aiamap.plot(axes=ax, cmap=aiacmap)
                    ax = axs[1]
                    aiamap.plot(axes=ax, cmap=aiacmap)
                    self._showaia = True
                except:
                    self._showaia = False

        if self._showaia:
            if self.opencontour is None:
                self.opencontour = False
        else:
            if self.opencontour is None:
                self.opencontour = True
        ## Plot EOVSA images as filled contour on top of the AIA image
        icmap = plt.get_cmap('RdYlBu')
        cts = []
        ## color map for spectra from the image series
        tcmap = plt.get_cmap('turbo')

        for s, sp in enumerate(rfreqs):
            data = rdata[s, ...]
            clvls = clevels * np.nanmax(data)
            rcmap = [icmap(self.freq_dist(self.cfreqs[s]))] * len(clvls)
            if self.opencontour:
                cts.append(ax.contour(mapx, mapy, data, levels=clvls,
                                      colors=rcmap,
                                      alpha=calpha))
            else:
                cts.append(ax.contourf(mapx, mapy, data, levels=clvls,
                                       colors=rcmap,
                                       alpha=calpha))
        ax.set_xlim(self.xlim)
        ax.set_ylim(self.ylim)
        for ax in axs[:2]:
            ax.set_xlabel('Solar-X [arcsec]')
            ax.set_ylabel('Solar-y [arcsec]')
            ax.set_title('')
        ax.text(0.02, 0.01,
                ' '.join(['AIA {:.0f} Å'.format(aiamap.wavelength.value),
                          aiamap.date.datetime.strftime('%Y-%m-%dT%H:%M:%S')]),
                ha='left',
                va='bottom',
                color='k', transform=ax.transAxes)
        ax.text(0.02, 0.05, ' '.join(['EOVSA     ', eodate.datetime.strftime('%Y-%m-%dT%H:%M:%S')]), ha='left',
                va='bottom',
                color='k', transform=ax.transAxes)

        divider = make_axes_locatable(axs[0])
        cax = divider.append_axes("right", size="8%", pad=0.08)
        cax.set_visible(False)

        divider = make_axes_locatable(axs[1])
        cax = divider.append_axes("right", size="8%", pad=0.08)

        ticks, bounds, vmax, vmin, freqmask = ql.get_colorbar_params(bdinfo)

        cb = colorbar.ColorbarBase(cax, norm=colors.Normalize(vmin=vmin, vmax=vmax), cmap=icmap,
                                   orientation='vertical', boundaries=bounds, spacing='proportional',
                                   ticks=ticks, format='%4.1f', alpha=calpha)

        for fbd_lo, fbd_hi in freqmask:
            if fbd_hi is not None:
                cax.axhspan(fbd_lo, fbd_hi, hatch='//', edgecolor='k', facecolor='#BBBBBB')
        plt.text(0.5, 1.05, 'MW', ha='center', va='bottom', transform=cax.transAxes, color='k', fontweight='normal')
        plt.text(0.5, 1.01, '[GHz]', ha='center', va='bottom', transform=cax.transAxes, color='k',
                 fontweight='normal')

        cax.xaxis.set_visible(False)
        cax.tick_params(axis="y", pad=-20., length=0, colors='k', labelsize=7)
        cax.axhline(vmin, xmin=1.0, xmax=1.2, color='k', clip_on=False)
        cax.axhline(vmax, xmin=1.0, xmax=1.2, color='k', clip_on=False)
        cax.text(1.25, 0.0, '{:.1f}'.format(vmin), fontsize=9, transform=cax.transAxes, va='center', ha='left')
        cax.text(1.25, 1.0, '{:.1f}'.format(vmax), fontsize=9, transform=cax.transAxes, va='center', ha='left')

        boxlines = []
        clkpnts = []

        for idx, ax in enumerate(axs[:2]):
            if idx == 0:
                c = 'g'
            elif idx == 1:
                c = 'b'
            else:
                c = 'k'
            line, = ax.plot([], [], '-', c=c, alpha=1.0)  # empty line
            boxlines.append(line)
            clkpnt, = ax.plot([], [], '+', c='white', alpha=0.7)  # empty line
            clkpnts.append(clkpnt)

        if ntim < 2:
            cplts = ['k']
        else:
            cplts = tcmap(np.linspace(0, 1, ntim))

        self.cplts = cplts
        self.ax_eospec = axs[-1]

        errobjs = initspecplot(self.ax_eospec, cplts)
        grids.tight_layout(fig)

        self.region = RegionSelector(clkpnts, boxlines, eofiles, errobjs, cfreqs=cfreqs, rms=None, wTmap=None)
        self.scatter_eospecs_fit = []
        self.scatter_eospecs = []

    def set_params(self, params):
        ssz = self.region.area  # source area in arcsec^2
        params.add('ssz', value=ssz, vary=False)  # pixel size in arcsec^2
        self.params = params

    def plot_components(self):
        ti = 0
        tb = self.region.errobjs[ti][0].get_ydata() * 1e6
        tb_ma = ma.masked_less_equal(tb, 0)
        freqghz = self.region.errobjs[0][0].get_xdata()
        # freqghz_ma = ma.masked_outside(freqghz, 1.0, 15.0)
        freqghz_ma = ma.masked_outside(freqghz, self.freqghz_bound[0], self.freqghz_bound[1])
        mask_fit = np.logical_or(freqghz_ma.mask, tb_ma.mask)
        freqghz_ma = ma.masked_array(freqghz, mask_fit)
        tb_ma = ma.masked_array(tb, mask_fit)

        # scatter_eospecs_fit.append(
        #     ax_spec.plot(freqghz_ma, tb_ma / 1.e6, marker='o', linestyle='', c=cplts[ti]))

        # flx_rms = rms
        tb_err = tb * 0.0
        tb_err[:] = 1.e6
        tb_err_ma = ma.masked_array(tb_err, tb_ma.mask)
        if len(self.scatter_eospecs_fit) == 0:
            for ti, cplt in enumerate(self.cplts):
                self.scatter_eospecs_fit.append(
                    self.ax_eospec.errorbar(freqghz_ma, tb_ma / 1.e6, yerr=tb_err_ma / 1.e6, marker='.', ms=1,
                                            linestyle='',
                                            c=cplt))
        else:
            for ti, cplt in enumerate(self.cplts):
                set_errorobj(freqghz_ma, tb_ma / 1.e6, self.scatter_eospecs_fit[ti], yerr=tb_err_ma / 1.e6)

    def fit(self):
        ti = 0
        tb = self.region.errobjs[ti][0].get_ydata() * 1e6
        tb_ma = ma.masked_less_equal(tb, 0)
        freqghz = self.region.errobjs[0][0].get_xdata()
        # freqghz_ma = ma.masked_outside(freqghz, 1.0, 15.0)
        freqghz_ma = ma.masked_outside(freqghz, self.freqghz_bound[0], self.freqghz_bound[1])
        mask_fit = np.logical_or(freqghz_ma.mask, tb_ma.mask)
        freqghz_ma = ma.masked_array(freqghz, mask_fit)
        tb_ma = ma.masked_array(tb, mask_fit)

        # scatter_eospecs_fit.append(
        #     ax_spec.plot(freqghz_ma, tb_ma / 1.e6, marker='o', linestyle='', c=cplts[ti]))

        # flx_rms = rms
        tb_err = tb * 0.1
        # tb_err[:] = 0.2e6
        tb_err_ma = ma.masked_array(tb_err, tb_ma.mask)
        if len(self.scatter_eospecs_fit) == 0:
            for ti, cplt in enumerate(self.cplts):
                self.scatter_eospecs_fit.append(
                    self.ax_eospec.errorbar(freqghz_ma, tb_ma / 1.e6, yerr=tb_err_ma / 1.e6, marker='.', ms=1,
                                            linestyle='', c=cplt))
        else:
            for ti, cplt in enumerate(self.cplts):
                set_errorobj(freqghz_ma, tb_ma / 1.e6, self.scatter_eospecs_fit[ti], yerr=tb_err_ma / 1.e6)

        mini = lmfit.Minimizer(mwspec2min_1src, self.params, fcn_args=(freqghz_ma.compressed(),),
                               fcn_kws={'tb': tb_ma.compressed(), 'tb_err': tb_err_ma.compressed()},
                               nan_policy='omit')
        method = 'nelder'
        # # method = 'differential_evolution'
        mi = mini.minimize(method=method)
        print(method + ' minimization results')
        print(lmfit.fit_report(mi.params))

        tb_fit = mwspec2min_1src(mi.params, freqghz)
        if len(self.scatter_eospecs) == 0:
            for ti, cplt in enumerate(self.cplts):
                self.scatter_eospecs.append(self.ax_eospec.plot(freqghz, tb_fit / 1.e6, linestyle='-', c=cplt))
        else:
            for ti, cplt in enumerate(self.cplts):
                self.scatter_eospecs[ti][0].set_data(freqghz, tb_fit / 1.e6)
